(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.indicators = {}));
})(this, (function (exports) { 'use strict';

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function _cvi(
        high, low,
        period, size = high.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period * 2 - 1) throw new Error("Out of range")

        const per = 2 / (period + 1);

        const lag = {
            size: period,
            index: 0,
            pushes: 0,
            vals: []
        };

        let val = high[0] - low[0];

        for (let i = 1; i < period * 2 - 1; ++i) {
            val = ((high[i] - low[i]) - val) * per + val;
            // Start ti_buffer_qpush
            // BUFFER = lag
            // VAL    = val
            lag.vals[lag.index] = val;
            lag.index = lag.index + 1;
            if (lag.index >= lag.size) lag.index = 0;
            // End ti_buffer_qpush
        }

        for (let i = period * 2 - 1; i < size; ++i) {
            val = ((high[i] - low[i]) - val) * per + val;

            const old = lag.vals[lag.index];

            output.push(100.0 * (val - old) / old);
            // Start ti_buffer_qpush
            // BUFFER = lag
            // VAL    = val
            lag.vals[lag.index] = val;
            lag.index = lag.index + 1;
            if (lag.index >= lag.size) lag.index = 0;
            // End ti_buffer_qpush

        }

        return output
    }

    var _cvi_1 = _cvi;

    /**
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} period 
     * @param {number} [size]
     * @returns [upper_band, lower_band, middle_point]
     */
    async function abands(
        high, low,
        close, period,
        size = high.length
    ) {

        const upper_band = [];
        const lower_band = [];
        const middle_point = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        const per = 1. / period;

        const buffer_high = {
            size: period,
            index: 0,
            pushes: 0,
            sum: 0,
            vals: []
        };
        const buffer_low = {
            size: period,
            index: 0,
            pushes: 0,
            sum: 0,
            vals: []
        };

        let close_sum = 0;

        for (let i = 0; i < period; ++i) {
            // Start MULT
            // MULT(i) (4. * (high[i] - low[i]) / (high[i] + low[i]))
            const mult = (4. * (high[i] - low[i]) / (high[i] + low[i]));
            // End MULT

            const high_val = (1. + mult) * high[i];

            // Start ti_buffer_push
            // BUFFER = buffer_high
            // VAL    = high_val
            // ti_buffer_push(buffer_high, high_val)
            if (buffer_high.pushes >= buffer_high.size) {
                buffer_high.sum -= buffer_high.vals[buffer_high.index];
            }

            buffer_high.sum += high_val;
            buffer_high.vals[buffer_high.index] = high_val;
            buffer_high.pushes += 1;
            buffer_high.index = (buffer_high.index + 1);
            if (buffer_high.index >= buffer_high.size) buffer_high.index = 0;
            // End ti_buffer_push

            const low_val = (1. - mult) * low[i];

            // Start ti_buffer_push
            // BUFFER = buffer_low
            // VAL    = low_val
            // ti_buffer_push(buffer_low, low_val)
            if (buffer_low.pushes >= buffer_low.size) {
                buffer_low.sum -= buffer_low.vals[buffer_low.index];
            }

            buffer_low.sum += low_val;
            buffer_low.vals[buffer_low.index] = low_val;
            buffer_low.pushes += 1;
            buffer_low.index = (buffer_low.index + 1);
            if (buffer_low.index >= buffer_low.size) buffer_low.index = 0;
            // End ti_buffer_push

            close_sum += close[i];
        }

        upper_band.push(buffer_high.sum * per);
        lower_band.push(buffer_low.sum * per);
        middle_point.push(close_sum * per);

        for (let i = period; i < size; ++i) {
            // Start MULT
            const mult = (4. * (high[i] - low[i]) / (high[i] + low[i]));
            // End MULT

            const high_val = (1. + mult) * high[i];
            // Start ti_buffer_push
            // BUFFER = buffer_high
            // VAL    = high_val
            // ti_buffer_push(buffer_high, high_val)
            if (buffer_high.pushes >= buffer_high.size) {
                buffer_high.sum -= buffer_high.vals[buffer_high.index];
            }

            buffer_high.sum += high_val;
            buffer_high.vals[buffer_high.index] = high_val;
            buffer_high.pushes += 1;
            buffer_high.index = (buffer_high.index + 1);
            if (buffer_high.index >= buffer_high.size) buffer_high.index = 0;
            // End ti_buffer_push

            const low_val = (1. - mult) * low[i];

            // Start ti_buffer_push
            // BUFFER = buffer_low
            // VAL    = low_val
            // ti_buffer_push(buffer_low, low_val)
            if (buffer_low.pushes >= buffer_low.size) {
                buffer_low.sum -= buffer_low.vals[buffer_low.index];
            }

            buffer_low.sum += low_val;
            buffer_low.vals[buffer_low.index] = low_val;
            buffer_low.pushes += 1;
            buffer_low.index = (buffer_low.index + 1);
            if (buffer_low.index >= buffer_low.size) buffer_low.index = 0;
            // End ti_buffer_push

            close_sum += close[i] - close[i - period];

            upper_band.push(buffer_high.sum * per);
            lower_band.push(buffer_low.sum * per);
            middle_point.push(close_sum * per);
        }

        return [upper_band, lower_band, middle_point]
    }

    var abands_1 = abands;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number[]} volume 
     * @param {number} [size]
     * @returns 
     */
    async function ad(
        high, low,
        close, volume,
        size = close.length
    ) {

        const output = [];

        let sum = 0;

        for (let index = 0; index < size; ++index) {
            const hl = (high[index] - low[index]);
            if (hl != 0.0) {
                sum += (close[index] - low[index] - high[index] + close[index]) / hl * volume[index];
            }
            output[index] = sum;
        }

        return output
    }

    var ad_1 = ad;

    /**
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number[]} volume 
     * @param {number} short_period 
     * @param {number} long_period 
     * @param {number} [size]
     * @returns 
     */
    async function adosc(
        high, low,
        close, volume,
        short_period, long_period,
        size = close.length
    ) {

        const start = long_period - 1;

        // if (short_period < 1) throw new Error("Invalid Options")
        // if (long_period < short_period) throw new Error("Invalid Options")
        // if (size <= long_period - 1) throw new Error("Out of range")

        const short_per = 2 / (short_period + 1);
        const long_per = 2 / (long_period + 1);

        const output = [];

        let sum = 0;
        let short_ema = 0;
        let long_ema = 0;

        for (let index = 0; index < size; ++index) {

            const hl = (high[index] - low[index]);
            if (hl != 0.0) {
                sum += (close[index] - low[index] - high[index] + close[index]) / hl * volume[index];
            }

            if (index == 0) {
                short_ema = sum;
                long_ema = sum;
            } else {
                short_ema = (sum - short_ema) * short_per + short_ema;
                long_ema = (sum - long_ema) * long_per + long_ema;
            }

            if (index >= start) {
                // output[output.length] = short_ema - long_ema
                output.push(short_ema - long_ema);
            }
        }

        return output
    }

    var adosc_1 = adosc;

    /**
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function adx(
        high, low,
        period, size = high.length
    ) {

        const output = [];

        // if (period < 2) throw new Error("Invalid Options")
        // if (size <= (period - 1) * 2) throw new Error("Out of range")

        const per = ((period - 1)) / period;
        const invper = 1.0 / (period);

        let dmup = 0;
        let dmdown = 0;

        for (let index = 1; index < period; ++index) {

            let dp = high[index] - high[index - 1];
            let dm = low[index - 1] - low[index];

            if (dp < 0) {
                dp = 0;
            } else if (dp > dm) {
                dm = 0;
            }

            if (dm < 0) {
                dm = 0;
            } else if (dm > dp) {
                dp = 0;
            }


            dmup += dp;
            dmdown += dm;
        }

        let adx = 0.0;

        let di_up = dmup;
        let di_down = dmdown;
        let dm_diff = Math.abs(di_up - di_down);
        let dm_sum = di_up + di_down;
        let dx = dm_diff / dm_sum * 100;

        adx += dx;

        for (let index = period; index < size; ++index) {

            let dp = high[index] - high[index - 1];
            let dm = low[index - 1] - low[index];

            if (dp < 0) {
                dp = 0;
            } else if (dp > dm) {
                dm = 0;
            }

            if (dm < 0) {
                dm = 0;
            } else if (dm > dp) {
                dp = 0;
            }


            dmup = dmup * per + dp;
            dmdown = dmdown * per + dm;

            di_up = dmup;
            di_down = dmdown;
            dm_diff = Math.abs(di_up - di_down);
            dm_sum = di_up + di_down;
            dx = dm_diff / dm_sum * 100;

            if (index - period < period - 2) {
                adx += dx;
            } else if (index - period == period - 2) {
                adx += dx;
                // output[output.length] = adx * invper
                output.push(adx * invper);
            } else {
                adx = adx * per + dx;
                // output[output.length] = adx * invper
                output.push(adx * invper);
            }
        }

        return output
    }

    var adx_1 = adx;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function adxr(
        high, low,
        period, size = high.length
    ) {

        const output = [];

        // if (period < 2) throw new Error("Invalid Options")
        // if (size <= (period-1) * 3) throw new Error("Out of Range")

        const per = (period - 1) / (period);
        const invper = 1.0 / (period);

        let dmup = 0;
        let dmdown = 0;

        for (let i = 1; i < period; ++i) {

            // Start CALC_DIRECTION
            let dp = high[i] - high[i - 1];
            let dm = low[i - 1] - low[i];

            if (dp < 0) {
                dp = 0;
            } else if (dp > dm) {
                dm = 0;
            }

            if (dm < 0) {
                dm = 0;
            } else if (dm > dp) {
                dp = 0;
            }
            // End CALC_DIRECTION

            dmup += dp;
            dmdown += dm;
        }

        let adx = 0.0;

        let di_up = dmup;
        let di_down = dmdown;
        let dm_diff = Math.abs(di_up - di_down);
        let dm_sum = di_up + di_down;
        let dx = dm_diff / dm_sum * 100;

        adx += dx;

        // Start ti_buffer_new
        const adxr = {
            size: period - 1,
            pushes: 0,
            index: 0,
            vals: [],
        };
        // End ti_buffer_new

        const first_adxr = (period - 1) * 3;

        for (let i = period; i < size; ++i) {

            // Start CALC_DIRECTION
            let dp = high[i] - high[i - 1];
            let dm = low[i - 1] - low[i];

            if (dp < 0) {
                dp = 0;
            } else if (dp > dm) {
                dm = 0;
            }

            if (dm < 0) {
                dm = 0;
            } else if (dm > dp) {
                dp = 0;
            }
            // End CALC_DIRECTION

            dmup = dmup * per + dp;
            dmdown = dmdown * per + dm;

            di_up = dmup;
            di_down = dmdown;
            dm_diff = Math.abs(di_up - di_down);
            dm_sum = di_up + di_down;
            dx = dm_diff / dm_sum * 100;

            if (i - period < period - 2) {
                adx += dx;
            } else if (i - period == period - 2) {
                adx += dx;
                // ## Start ti_buffer_qpush
                // BUFFER   = adxr
                // VAL      = adx * invper
                adxr.vals[adxr.index] = adx * invper;
                adxr.index = adxr.index + 1;
                if (adxr.index >= adxr.size) adxr.index = 0;
                // ## End ti_buffer_qpush
            } else {
                adx = adx * per + dx;

                if (i >= first_adxr) {
                    // ## Start ti_buffer_get
                    // BUFFER = adxr
                    // VAL    = 1
                    // ((BUFFER)->vals[((BUFFER)->index + (BUFFER)->size - 1 + (VAL)) % (BUFFER)->size])
                    const a = adxr.vals[adxr.index];
                    const b = (adxr.size - 1) + 1; // + 1 = VAL
                    const c = adxr.size;
                    const buffer_get = a + b % c;
                    // ## End ti_buffer_get
                    output.push((0.5 * (adx * invper + buffer_get)));
                }

                // ## Start ti_buffer_qpush
                adxr.vals[adxr.index] = adx * invper;
                adxr.index = adxr.index + 1;
                if (adxr.index >= adxr.size) adxr.index = 0;
                // ## End ti_buffer_qpush
            }
        }

        return output
    }

    var adxr_1 = adxr;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} offset 
     * @param {number} sigma 
     * @param {number} [size]
     * @returns 
     */
     async function alma(
        source, period,
        offset, sigma,
        size = source.length
    ) {

        const output = [];

        // validate options
        // if (period < 1) throw new Error("Invalid Options")
        // if (sigma <= 0) throw new Error("Invalid Options")
        // if ((offset < 0) || (offset > 1)) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        const weights = [];

        const m = Math.floor(offset * (period - 1));
        const s = period / sigma;

        let norm = 0;

        for (let i = 0; i < period; i++) {
            weights[i] = Math.exp(-1 * Math.pow(i - m, 2) / (2 * Math.pow(s, 2)));
            norm += weights[i];
        }

        for (let i = 0; i < period; i++) {
            weights[i] /= norm;
        }

        for (let i = period - 1; i < size; i++) {
            let sum = 0;
            for (let j = 0; j < period; j++) {
                sum += source[i - period + j + 1] * weights[j];
            }
            output.push(sum);
        }

        return output
    }

    var alma_1 = alma;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} [size]
     * @returns 
     */
    async function ao(
        high, low,
        size = high.length
    ) {

        const period = 34;

        const output = [];

        // if (size <= 33) throw new Error("Out of range")

        let sum34 = 0;
        let sum5 = 0;
        const per34 = 1.0 / 34.0;
        const per5 = 1.0 / 5.0;

        for (let index = 0; index < 34; ++index) {
            const hl = 0.5 * (high[index] + low[index]);

            sum34 += hl;

            if (index >= 29) {
                sum5 += hl;
            }
        }

        output.push(per5 * sum5 - per34 * sum34);

        for (let index = period; index < size; ++index) {
            const hl = 0.5 * (high[index] + low[index]);

            sum34 += hl;
            sum5 += hl;

            sum34 -= 0.5 * (high[index - 34] + low[index - 34]);
            sum5 -= 0.5 * (high[index - 5] + low[index - 5]);

            // output[output.length] = per5 * sum5 - per34 * sum34
            output.push(per5 * sum5 - per34 * sum34);
        }

        return output
    }

    var ao_1 = ao;

    /**
     * 
     * @param {number[]} source 
     * @param {number} short_period 
     * @param {number} long_period 
     * @param {number} [size]
     * @returns 
     */
     async function apo(
        source, short_period, long_period,
        size = source.length
    ) {

        const output = [];

        // if (short_period < 1) throw new Error("Invalid Options")
        // if (long_period < 2) throw new Error("Invalid Options")
        // if (long_period < short_period) throw new Error("Invalid Options")
        // if (size <= 1) throw new Error("Out of range")

        const short_per = 2 / (short_period + 1);
        const long_per = 2 / (long_period + 1);

        let short_ema = source[0];
        let long_ema = source[0];

        for (let index = 1; index < size; ++index) {

            short_ema = (source[index] - short_ema) * short_per + short_ema;
            long_ema = (source[index] - long_ema) * long_per + long_ema;

            const out = short_ema - long_ema;

            // output[output.length] = out
            output.push(out);
        }

        return output
    }

    var apo_1 = apo;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} period 
     * @param {number} [size]
     * @returns First output = Aroon Down, Second output = Aroon Up
     */
     async function aroon(
        high, low,
        period, size = high.length
    ) {

        const adown = [];
        const aup = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        const scale = 100.0 / period;
        let trail = 0;
        let maxi = -1;
        let mini = -1;
        let max = high[0];
        let min = low[0];
        let bar;

        let j;
        for (let index = period; index < size; ++index, ++trail) {

            // Maintain highest
            bar = high[index];

            if (maxi < trail) {
                maxi = trail;
                max = high[maxi];
                j = trail;

                while (++j <= index) {
                    bar = high[j];

                    if (bar >= max) {
                        max = bar;
                        maxi = j;
                    }
                }

            } else if (bar >= max) {
                maxi = index;
                max = bar;
            }

            // Maintain lowest
            bar = low[index];
            if (mini < trail) {
                mini = trail;
                min = low[mini];
                j = trail;
                while (++j <= index) {
                    bar = low[j];
                    if (bar <= min) {
                        min = bar;
                        mini = j;
                    }
                }

            } else if (bar <= min) {
                mini = index;
                min = bar;
            }

            adown[adown.length] = (period - (index - mini)) * scale;
            aup[aup.length] = (period - (index - maxi)) * scale;
        }

        return [adown, aup]
    }

    var aroon_1 = aroon;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function aroonosc(
        high, low,
        period, size = high.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        const scale = 100.0 / period;

        let trail = 0;
        let maxi = -1;
        let mini = -1;
        let max = high[0];
        let min = low[0];

        let j;
        for (let index = period; index < size; ++index, ++trail) {

            // Maintain highest
            let bar = high[index];
            if (maxi < trail) {
                maxi = trail;
                max = high[maxi];
                j = trail;
                while (++j <= index) {
                    bar = high[j];
                    if (bar >= max) {
                        max = bar;
                        maxi = j;
                    }
                }

            } else if (bar >= max) {
                maxi = index;
                max = bar;
            }


            // Maintain lowest
            bar = low[index];
            if (mini < trail) {
                mini = trail;
                min = low[mini];
                j = trail;
                while (++j <= index) {
                    bar = low[j];
                    if (bar <= min) {
                        min = bar;
                        mini = j;
                    }
                }

            } else if (bar <= min) {
                mini = index;
                min = bar;
            }

            // output[output.length] = (maxi - mini) * scale
            output.push((maxi - mini) * scale);
        }

        return output
    }

    var aroonosc_1 = aroonosc;

    /**
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function atr(
        high, low, close,
        period, size = high.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid options")
        // if (size <= period - 1) throw new Error("Out of range")

        const per = 1.0 / period;

        let sum = 0;
        let truerange;

        sum += high[0] - low[0];

        for (let i = 1; i < period; ++i) {

            // Start CALC_TRUERANGE()
            const l = low[i];
            const h = high[i];
            const c = close[i - 1];

            const ych = Math.abs(h - c);
            const ycl = Math.abs(l - c);

            let v = h - l;

            if (ych > v) v = ych;
            if (ycl > v) v = ycl;
            truerange = v;
            // End CALC_TRUERANGE()

            sum += truerange;
        }

        let val = sum / period;

        // output[output.length] = val
        output.push(val);

        for (let index = period; index < size; ++index) {
            // CALC TRUE RANGE
            const l = low[index];
            const h = high[index];
            const c = close[index - 1];

            const ych = Math.abs(h - c);
            const ycl = Math.abs(l - c);

            let v = h - l;

            if (ych > v) v = ych;
            if (ycl > v) v = ycl;
            truerange = v;
            // CALC TRUE RANGE

            val = (truerange - val) * per + val;

            // output[output.length] = val
            output.push(val);
        }

        return output
    }

    var atr_1 = atr;

    /**
     * @param {number[]} open 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} [size]
     * @returns 
     */
    async function avgprice(
        open, high,
        low, close,
        size = open.length
    ) {

        const output = [];

        for (let index = 0; index < size; ++index) {
            output.push((open[index] + high[index] + low[index] + close[index]) * 0.25);
        }

        return output
    }

    var avgprice_1 = avgprice;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} stddev 
     * @param {number} [size]
     * @returns [Lower, Middle, Upper]
     */
     async function bbands(
        source, period, stddev,
        size = source.length
    ) {

        const lower = [];
        const middle = [];
        const upper = [];

        const scale = 1.0 / period;

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let sum = 0;
        let sum2 = 0;

        for (let i = 0; i < period; ++i) {
            sum += source[i];
            sum2 += source[i] * source[i];
        }

        let sd = Math.sqrt(sum2 * scale - (sum * scale) * (sum * scale));

        const calc_middle = sum * scale;
        lower.push(calc_middle - stddev * sd);
        upper.push(calc_middle + stddev * sd);
        middle.push(calc_middle);

        for (let i = period; i < size; ++i) {
            sum += source[i];
            sum2 += source[i] * source[i];

            sum -= source[i - period];
            sum2 -= source[i - period] * source[i - period];

            sd = Math.sqrt(sum2 * scale - (sum * scale) * (sum * scale));

            const calc_middle = sum * scale;
            upper.push(calc_middle + stddev * sd);
            lower.push(calc_middle - stddev * sd);
            middle.push(calc_middle);
        }

        return [lower, middle, upper]
    }

    var bbands_1 = bbands;

    /**
     * 
     * @param {number[]} open 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} [size]
     * @returns 
     */
     async function bop(
        open, high,
        low, close,
        size = open.length
    ) {

        const output = [];

        for (let index = 0; index < size; ++index) {
            const hl = high[index] - low[index];

            if (hl <= 0.0) {
                output[index] = 0;
            } else {
                output[index] = (close[index] - open[index]) / hl;
            }
        }

        return output
    }

    var bop_1 = bop;

    let ti_buffer$2 = class ti_buffer {

    	size;
    	pushes;
    	index;
    	sum;
    	vals;

    	constructor(size) {
    		this.size = size;
    		this.pushes = 0;
    		this.index = 0;
    		this.sum = 0;
    		this.vals = [];
    	}

    	static new(size) {
    		return new ti_buffer(size);
    	}

    	push(val) {
    		if (this.pushes >= this.size) {
    			this.sum -= this.vals[this.index];
    		}

    		this.sum += val;
    		this.vals[this.index] = val;
    		this.pushes += 1;
    		this.index = (this.index + 1) % this.size;
    	}

    	qpush(val) {
    		this.vals[this.index] = val;
    		this.index = (this.index + 1) % this.size;
    	}

    	get(val) {
    		return this.vals[(this.index + this.size - 1 + val) % this.size];
    	}
    };

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function cci(
        high, low,
        close, period,
        size = high.length
    ) {

        const scale = 1.0 / period;

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= (period-1) * 2) throw new Error("Out of range")

        const output = [];

        const sum = new ti_buffer$2(period);

        let i, j;
        for (i = 0; i < size; ++i) {

            // Start TYPEPRICE
            // ((high[(INDEX)] + low[(INDEX)] + close[(INDEX)]) * (1.0/3.0))
            const today = ((high[(i)] + low[(i)] + close[(i)]) * (1.0 / 3.0));
            // End TYPEPRICE

            sum.push(today);
            const avg = sum.sum * scale;

            if (i >= period * 2 - 2) {
                let acc = 0;
                for (j = 0; j < period; ++j) {
                    acc += Math.abs(avg - sum.vals[j]);
                }

                let cci = acc * scale;
                cci *= .015;
                cci = (today - avg) / cci;
                output.push(cci);
            }
        }

        return output
    }

    var cci_1 = cci;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} period 
     * @param {number} coef 
     * @param {number} [size]
     * @returns [ce_high, ce_low]
     */
    async function ce(
        high, low,
        close, period, coef,
        size = high.length
    ) {

        const ce_high = [];
        const ce_low = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let atr = high[0] - low[0];

        let truerange;
        let val;

        let HP = high[0];
        let HP_idx = 0;
        let LP = low[0];
        let LP_idx = 0;

        for (let i = 1; i < period; ++i) {
            // Start CALC_TRUERANGE()
            const l = low[i];
            const h = high[i];
            const c = close[i - 1];

            const ych = Math.abs(h - c);
            const ycl = Math.abs(l - c);

            let v = h - l;

            if (ych > v) v = ych;
            if (ycl > v) v = ycl;
            truerange = v;
            // End CALC_TRUERANGE()

            atr += truerange;

            if (HP <= (val = high[i])) {
                HP = val;
                HP_idx = i;
            }
            if (LP >= (val = low[i])) {
                LP = val;
                LP_idx = i;
            }
        }

        atr /= period;

        const smth = (period - 1) / period;
        const per = 1 / period;

        ce_high.push(HP - coef * atr);
        ce_low.push(LP + coef * atr);

        for (let i = period; i < size; ++i) {
            // Start CALC_TRUERANGE()
            const l = low[i];
            const h = high[i];
            const c = close[i - 1];

            const ych = Math.abs(h - c);
            const ycl = Math.abs(l - c);

            let v = h - l;

            if (ych > v) v = ych;
            if (ycl > v) v = ycl;
            truerange = v;
            // End CALC_TRUERANGE()
            atr = atr * smth + truerange * per;

            if (HP <= (val = high[i])) {
                HP = val;
                HP_idx = i;
            } else if (HP_idx == i - period) {
                HP = high[i - period + 1];
                HP_idx = i - period + 1;

                for (let j = i - period + 2; j <= i; ++j) {
                    if (HP <= (val = high[j])) {
                        HP = val;
                        HP_idx = j;
                    }
                }
            }

            if (LP >= (val = low[i])) {
                LP = val;
                LP_idx = i;
            } else if (LP_idx == i - period) {
                LP = low[i - period + 1];
                LP_idx = i - period + 1;

                for (let j = i - period + 2; j <= i; ++j) {
                    if (LP >= (val = low[j])) {
                        LP = val;
                        LP_idx = j;
                    }
                }
            }

            ce_high.push(HP - coef * atr);
            ce_low.push(LP + coef * atr);
        }

        return [ce_high, ce_low]
    }

    var ce_1 = ce;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number[]} volume 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function cmf(
        high, low,
        close, volume,
        period, size = high.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let period_volume = 0;
        let period_ad_sum = 0;

        for (let i = 0; i < period - 1; ++i) {
            // Start CHAIKIN_AD
            period_ad_sum += (high[i] - low[i] ? volume[i] * ((close[i] - low[i]) - (high[i] - close[i])) / (high[i] - low[i]) : 0.);
            // End CHAIKIN_AD

            period_volume += volume[i];
        }

        for (let i = period - 1; i < size; ++i) {
            // Start CHAIKIN_AD
            period_ad_sum += (high[i] - low[i] ? volume[i] * ((close[i] - low[i]) - (high[i] - close[i])) / (high[i] - low[i]) : 0.);
            // End CHAIKIN_AD
            period_volume += volume[i];

            output.push(period_ad_sum / period_volume);

            // Start CHAIKIN_AD
            // i = i-period+1
            period_ad_sum -= (high[i - period + 1] - low[i - period + 1] ? volume[i - period + 1] * ((close[i - period + 1] - low[i - period + 1]) - (high[i - period + 1] - close[i - period + 1])) / (high[i - period + 1] - low[i - period + 1]) : 0.);
            // End CHAIKIN_AD
            period_volume -= volume[i - period + 1];
        }

        return output
    }

    var cmf_1 = cmf;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function cmo(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid options")
        // if (size <= period) throw new Error("Out of range")

        let up_sum = 0;
        let down_sum = 0;

        for (let i = 1; i <= period; ++i) {
            // Start UPWARD
            // up_sum += UPWARD(i)
            up_sum += (source[(i)] > source[(i) - 1] ? source[(i)] - source[(i) - 1] : 0);
            // End UPWARD

            // Start DOWNWARD
            // down_sum += DOWNWARD(i)
            down_sum += (source[(i)] < source[(i) - 1] ? source[(i) - 1] - source[(i)] : 0);
            // End DOWNWARD
        }

        output.push(100 * (up_sum - down_sum) / (up_sum + down_sum));

        for (let i = period + 1; i < size; ++i) {
            // i - period
            up_sum -= (source[(i - period)] > source[(i - period) - 1] ? source[(i - period)] - source[(i - period) - 1] : 0);
            down_sum -= (source[(i - period)] < source[(i - period) - 1] ? source[(i - period) - 1] - source[(i - period)] : 0);

            // i
            up_sum += (source[(i)] > source[(i) - 1] ? source[(i)] - source[(i) - 1] : 0);
            down_sum += (source[(i)] < source[(i) - 1] ? source[(i) - 1] - source[(i)] : 0);

            output.push(100 * (up_sum - down_sum) / (up_sum + down_sum));
        }

        return output
    }

    var cmo_1 = cmo;

    /**
     * @param {number[]} data 
     * @param {number} period1 
     * @param {number} period2 
     * @returns 
     */
    async function copp(data, period1, period2) {

        const roc1 = new Array(data.length);
        const roc2 = new Array(data.length);
        const coppock = new Array(data.length);

        for (let i = 0; i < data.length; i++) {
            if (i < period1 - 1) {
                roc1[i] = null;
            } else {
                roc1[i] = (data[i] - data[i - period1]) / data[i - period1] * 100;
            }
            if (i < period2 - 1) {
                roc2[i] = null;
            } else {
                roc2[i] = (data[i] - data[i - period2]) / data[i - period2] * 100;
            }
            coppock[i] = (roc1[i] + roc2[i]) * 10;
        }

        return coppock;
    }

    var copp_1 = copp;

    /**
     * @param {number[]} a 
     * @param {number[]} b 
     * @param {number} [size]
     * @returns 
     */
    async function crossany(a, b, size = a.length) {
        const output = [];

        for (let i = 0; i < size; ++i) {
            output.push((a[i] > b[i] && a[i - 1] <= b[i - 1])
                || (a[i] < b[i] && a[i - 1] >= b[i - 1])
            );
        }

        return output
    }

    var crossany_1 = crossany;

    /**
     * @param {number[]} a 
     * @param {number[]} b 
     * @param {number} [size]
     * @returns 
     */
    async function crossover(a, b, size = a.length) {

        const output = [];

        for (let i = 0; i < size; ++i) {
            output.push(a[i] > b[i] && a[i - 1] <= b[i - 1]);
        }

        return output
    }

    var crossover_1 = crossover;

    /**
     * 
     * @param {number[]} seriesA 
     * @param {number} number 
     * @returns 
     */
     async function crossOverNumber(seriesA, number) {

        const output = [];

        for (let index = 0; index < seriesA.length; index++) {
            const current = seriesA[index];
            const pre = seriesA[index - 1];

            if (
                current > number &&
                pre <= number
            ) {
                output.push(true);
            } else {
                output.push(false);
            }
        }

        return output
    }

    var crossOverNumber_1 = crossOverNumber;

    /**
     * 
     * @param {number[]} seriesA 
     * @param {number} number 
     * @returns 
     */
    async function crossUnderNumber(seriesA, number) {

        const output = [];

        for (let index = 0; index < seriesA.length; index++) {
            const current = seriesA[index];
            const pre = seriesA[index - 1];

            if (
                current < number &&
                pre >= number
            ) {
                output.push(true);
            } else {
                output.push(false);
            }
        }

        return output
    }

    var crossUnderNumber_1 = crossUnderNumber;

    let ti_buffer$1 = class ti_buffer {

    	size;
    	pushes;
    	index;
    	sum;
    	vals;

    	constructor(size) {
    		this.size = size;
    		this.pushes = 0;
    		this.index = 0;
    		this.sum = 0;
    		this.vals = [];
    	}

    	static new(size) {
    		return new ti_buffer(size);
    	}

    	push(val) {
    		if (this.pushes >= this.size) {
    			this.sum -= this.vals[this.index];
    		}

    		this.sum += val;
    		this.vals[this.index] = val;
    		this.pushes += 1;
    		this.index = (this.index + 1) % this.size;
    	}

    	qpush(val) {
    		this.vals[this.index] = val;
    		this.index = (this.index + 1) % this.size;
    	}

    	get(val) {
    		return this.vals[(this.index + this.size - 1 + val) % this.size];
    	}
    };

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function cvi(
        high, low,
        period, size = high.length
    ) {

        const output = [];

        const per = 2 / (period + 1);

        const lag = new ti_buffer$1(period);

        let val = high[0] - low[0];

        let i;
        for (i = 1; i < period * 2 - 1; ++i) {
            val = ((high[i] - low[i]) - val) * per + val;

            lag.qpush(val);
        }

        for (i = period * 2 - 1; i < size; ++i) {
            val = ((high[i] - low[i]) - val) * per + val;

            const old = lag.vals[lag.index];

            output.push(100.0 * (val - old) / old);

            lag.qpush(val);
        }

        return output
    }

    var cvi_1 = cvi;

    /**
     * 
     * @param {number[]} highs 
     * @param {number[]} lows 
     * @param {number} period 
     * @returns [upper, middle, lower]
     */
     async function dc(highs, lows, period) {

        const upper = [];
        const lower = [];
        const middle = [];

        for (let i = period - 1; i < highs.length; i++) {
            upper.push(Math.max(...highs.slice(i - period + 1, i + 1)));
            lower.push(Math.min(...lows.slice(i - period + 1, i + 1)));
            middle.push((upper[upper.length - 1] + lower[lower.length - 1]) / 2);
        }

        return [upper, middle, lower]
    }

    var dc_1 = dc;

    /**
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function decay(
        source, period,
        size = source.length
    ) {

        const output = [];

        const scale = 1.0 / period;

        output.push(source[0]);

        for (let i = 1; i < size; ++i) {
            const d = output[output.length - 1] - scale;
            output.push(source[i] > d ? source[i] : d);
        }

        return output
    }

    var decay_1 = decay;

    /**
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function dema(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= (period-1) * 2) throw new Error("Out of range")

        const per = 2 / (period + 1);
        const per1 = 1.0 - per;

        /*Calculate EMA(input)*/
        let ema = source[0];

        /*Calculate EMA(EMA(input))*/
        let ema2 = ema;

        for (let i = 0; i < size; ++i) {
            ema = ema * per1 + source[i] * per;

            if (i == period - 1) {
                ema2 = ema;
            }

            if (i >= period - 1) {
                ema2 = ema2 * per1 + ema * per;

                if (i >= (period - 1) * 2) {
                    output.push(ema * 2 - ema2);
                }
            }
        }

        return output
    }

    var dema_1 = dema;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} period 
     * @param {number} [size]
     * @returns [Plus DI, Minus DI]
     */
     async function di(
        high, low, close,
        period, size = high.length
    ) {

        const plus_di = [];
        const minus_di = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        const per = (period - 1) / (period);

        let atr = 0;
        let dmup = 0;
        let dmdown = 0;

        for (let i = 1; i < period; ++i) {

            // let truerange
            // Start CALC_TRUERANGE()
            const l = low[i];
            const h = high[i];
            const c = close[i - 1];
            const ych = Math.abs(h - c);
            const ycl = Math.abs(l - c);
            let v = h - l;
            if (ych > v) v = ych;
            if (ycl > v) v = ycl;
            const truerange = v;
            // End CALC_TRUERANGE()
            atr += truerange;

            // Start CALC_DIRECTION
            let dp = high[i] - high[i - 1];
            let dm = low[i - 1] - low[i];

            if (dp < 0) {
                dp = 0;
            } else if (dp > dm) {
                dm = 0;
            }

            if (dm < 0) {
                dm = 0;
            } else if (dm > dp) {
                dp = 0;
            }
            // END CALC_DIRECTION

            dmup += dp;
            dmdown += dm;
        }


        plus_di.push(100.0 * dmup / atr);
        minus_di.push(100.0 * dmdown / atr);

        for (let i = period; i < size; ++i) {
            // let truerange
            // Start CALC_TRUERANGE()
            const l = low[i];
            const h = high[i];
            const c = close[i - 1];
            const ych = Math.abs(h - c);
            const ycl = Math.abs(l - c);
            let v = h - l;
            if (ych > v) v = ych;
            if (ycl > v) v = ycl;
            const truerange = v;
            // End CALC_TRUERANGE()
            atr = atr * per + truerange;

            // Start CALC_DIRECTION
            let dp = high[i] - high[i - 1];
            let dm = low[i - 1] - low[i];

            if (dp < 0) {
                dp = 0;
            } else if (dp > dm) {
                dm = 0;
            }

            if (dm < 0) {
                dm = 0;
            } else if (dm > dp) {
                dp = 0;
            }
            // END CALC_DIRECTION


            dmup = dmup * per + dp;
            dmdown = dmdown * per + dm;

            plus_di.push(100.0 * dmup / atr);
            minus_di.push(100.0 * dmdown / atr);
        }

        return [plus_di, minus_di]
    }

    var di_1 = di;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} period 
     * @param {number} [size]
     * @returns [Plus DM, Minus DM]
     */
     async function dm(
        high, low,
        period, size = high.length
    ) {

        const plus_dm = [];
        const minus_dm = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        const per = (period - 1) / (period);

        let dmup = 0;
        let dmdown = 0;

        for (let i = 1; i < period; ++i) {
            // Start CALC_DIRECTION
            let dp = high[i] - high[i - 1];
            let dm = low[i - 1] - low[i];

            if (dp < 0) {
                dp = 0;
            } else if (dp > dm) {
                dm = 0;
            }

            if (dm < 0) {
                dm = 0;
            } else if (dm > dp) {
                dp = 0;
            }
            // END CALC_DIRECTION

            dmup += dp;
            dmdown += dm;
        }


        plus_dm.push(dmup);
        minus_dm.push(dmdown);

        for (let i = period; i < size; ++i) {
            // Start CALC_DIRECTION
            let dp = high[i] - high[i - 1];
            let dm = low[i - 1] - low[i];

            if (dp < 0) {
                dp = 0;
            } else if (dp > dm) {
                dm = 0;
            }

            if (dm < 0) {
                dm = 0;
            } else if (dm > dp) {
                dp = 0;
            }
            // END CALC_DIRECTION

            dmup = dmup * per + dp;
            dmdown = dmdown * per + dm;

            plus_dm.push(dmup);
            minus_dm.push(dmdown);
        }

        return [plus_dm, minus_dm]
    }

    var dm_1 = dm;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function dpo(
        source, period,
        size = source.length
    ) {

        const back = period / 2 + 1;

        const output = [];

        const scale = 1.0 / period;

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let sum = 0;

        for (let i = 0; i < period; ++i) {
            sum += source[i];
        }

        output.push(source[period - 1 - back] - (sum * scale));

        for (let i = period; i < size; ++i) {
            sum += source[i];
            sum -= source[i - period];
            output.push(source[i - back] - (sum * scale));
        }

        return output
    }

    var dpo_1 = dpo;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function dx(
        high, low,
        period, size = high.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        const per = (period - 1) / (period);

        let dmup = 0;
        let dmdown = 0;

        for (let i = 1; i < period; ++i) {

            // Start CALC_DIRECTION
            let dp = high[i] - high[i - 1];
            let dm = low[i - 1] - low[i];

            if (dp < 0) {
                dp = 0;
            } else if (dp > dm) {
                dm = 0;
            }

            if (dm < 0) {
                dm = 0;
            } else if (dm > dp) {
                dp = 0;
            }
            // End CALC_DIRECTION

            dmup += dp;
            dmdown += dm;
        }

        let di_up = dmup;
        let di_down = dmdown;
        let dm_diff = Math.abs(di_up - di_down);
        let dm_sum = di_up + di_down;
        let _dx = dm_diff / dm_sum * 100;

        output.push(_dx);

        for (let i = period; i < size; ++i) {

            // Start CALC_DIRECTION
            let dp = high[i] - high[i - 1];
            let dm = low[i - 1] - low[i];

            if (dp < 0) {
                dp = 0;
            } else if (dp > dm) {
                dm = 0;
            }

            if (dm < 0) {
                dm = 0;
            } else if (dm > dp) {
                dp = 0;
            }
            // End CALC_DIRECTION


            dmup = dmup * per + dp;
            dmdown = dmdown * per + dm;


            di_up = dmup;
            di_down = dmdown;
            dm_diff = Math.abs(di_up - di_down);
            dm_sum = di_up + di_down;
            _dx = dm_diff / dm_sum * 100;

            output.push(_dx);
        }

        return output
    }

    var dx_1 = dx;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function edecay(
        source, period,
        size = source.length
    ) {

        const output = [];

        const scale = 1.0 - 1.0 / period;

        output.push(source[0]);

        for (let i = 1; i < size; ++i) {
            const d = output[output.length - 1] * scale;
            output.push(source[i] > d ? source[i] : d);
        }

        return output
    }

    var edecay_1 = edecay;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function ema(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= 0) throw new Error("Out of range")

        const per = 2 / (period + 1);

        let val = source[0];
        output.push(val);

        for (let i = 1; i < size; ++i) {
            val = (source[i] - val) * per + val;
            output.push(val);
        }

        return output
    }

    var ema_1 = ema;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} volume 
     * @param {number} [size]
     * @returns 
     */
     async function emv(
        high, low, volume,
        size = high.length
    ) {

        const output = [];

        // if (size <= 1) throw new Error("Out of range")

        let last = (high[0] + low[0]) * 0.5;

        for (let i = 1; i < size; ++i) {
            const hl = (high[i] + low[i]) * 0.5;
            const br = volume[i] / 10000.0 / (high[i] - low[i]);

            output.push((hl - last) / br);
            last = hl;
        }

        return output
    }

    var emv_1 = emv;

    /**
     * 
     * @param {number[]} close 
     * @param {number[]} volume 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function fi(
        close, volume,
        period, size = close.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= 1) throw new Error("Out of range")

        const per = 2. / (period + 1.);

        let ema = volume[1] * (close[1] - close[0]);

        for (let i = 1; i < size; ++i) {
            ema = (volume[i] * (close[i] - close[i - 1]) - ema) * per + ema;
            output.push(ema);
        }

        return output
    }

    var fi_1 = fi;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} period 
     * @param {number} [size]
     * @returns [fisher, signal]
     */
     async function fisher(
        high, low,
        period, size = high.length
    ) {

        const fisher = [];
        const signal = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let trail = 0;
        let maxi = -1;
        let mini = -1;

        let max = (0.5 * (high[(0)] + low[(0)]));
        let min = (0.5 * (high[(0)] + low[(0)]));
        let val1 = 0.0;
        let bar;
        let fish = 0.0;

        let j;
        for (let i = period - 1; i < size; ++i, ++trail) {
            /* Maintain highest. */
            bar = (0.5 * (high[(i)] + low[(i)]));
            if (maxi < trail) {
                maxi = trail;
                max = (0.5 * (high[(maxi)] + low[(maxi)]));
                j = trail;

                while (++j <= i) {
                    bar = (0.5 * (high[(j)] + low[(j)]));
                    if (bar >= max) {
                        max = bar;
                        maxi = j;
                    }
                }

            } else if (bar >= max) {
                maxi = i;
                max = bar;
            }


            /* Maintain lowest. */
            bar = (0.5 * (high[(i)] + low[(i)]));
            if (mini < trail) {
                mini = trail;
                min = (0.5 * (high[(mini)] + low[(mini)]));
                j = trail;

                while (++j <= i) {
                    bar = (0.5 * (high[(j)] + low[(j)]));
                    if (bar <= min) {
                        min = bar;
                        mini = j;
                    }
                }

            } else if (bar <= min) {
                mini = i;
                min = bar;
            }

            let mm = max - min;
            if (mm == 0.0) mm = 0.001;
            val1 = 0.33 * 2.0 * (((0.5 * (high[(i)] + low[(i)])) - min) / (mm) - 0.5) + 0.67 * val1;
            if (val1 > 0.99) val1 = .999;
            if (val1 < -0.99) val1 = -.999;

            signal.push(fish);
            fish = 0.5 * Math.log((1.0 + val1) / (1.0 - val1)) + 0.5 * fish;
            fisher.push(fish);
        }

        return [fisher, signal]
    }

    var fisher_1 = fisher;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function fosc(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        // LINEAR_REGRESSION(size, input, period, output, period+1)
        // Start LINEAR_REGRESSION
        let x = 0;
        let x2 = 0;

        let y = 0;
        let xy = 0;

        // Start INIT()
        const p = (1.0 / (period));
        let tsf = 0;
        // End INIT()

        for (let i = 0; i < (period) - 1; ++i) {
            x += i + 1;
            x2 += (i + 1) * (i + 1);
            xy += (source)[i] * (i + 1);
            y += (source)[i];
        }

        x += (period);
        x2 += (period) * (period);

        const bd = 1.0 / ((period) * x2 - x * x);

        for (let i = (period) - 1; i < (size); ++i) {
            xy += (source)[i] * (period);
            y += (source)[i];

            const b = ((period) * xy - x * y) * bd;
            // Start FINAL()
            // ## forecast = period + 1
            const a = (y - b * x) * p;
            if (i >= (period)) {
                output.push(100 * (source[i] - tsf) / source[i]);
            }
            tsf = (a + b * (period + 1));
            // Start FINAL()

            xy -= y;
            y -= (source)[i - (period) + 1];
        }
        // End LINEAR_REGRESSION

        return output
    }

    var fosc_1 = fosc;

    class ti_buffer {

    	size;
    	pushes;
    	index;
    	sum;
    	vals;

    	constructor(size) {
    		this.size = size;
    		this.pushes = 0;
    		this.index = 0;
    		this.sum = 0;
    		this.vals = [];
    	}

    	static new(size) {
    		return new ti_buffer(size);
    	}

    	push(val) {
    		if (this.pushes >= this.size) {
    			this.sum -= this.vals[this.index];
    		}

    		this.sum += val;
    		this.vals[this.index] = val;
    		this.pushes += 1;
    		this.index = (this.index + 1) % this.size;
    	}

    	qpush(val) {
    		this.vals[this.index] = val;
    		this.index = (this.index + 1) % this.size;
    	}

    	get(val) {
    		return this.vals[(this.index + this.size - 1 + val) % this.size];
    	}
    }

    /**
     * 
     * @param {number[]} input 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function hma(input, period, size = input.length) {

        const output = [];

        const period2 = Math.floor(period / 2);
        const periodsqrt = Math.floor(Math.sqrt(period));

        const weights = period * (period + 1) / 2;
        const weights2 = period2 * (period2 + 1) / 2;
        const weightssqrt = periodsqrt * (periodsqrt + 1) / 2;

        let sum = 0; /* Flat sum of previous numbers. */
        let weight_sum = 0; /* Weighted sum of previous numbers. */

        let sum2 = 0;
        let weight_sum2 = 0;

        let sumsqrt = 0;
        let weight_sumsqrt = 0;

        /* Setup up the WMA(period) and WMA(period/2) on the input. */
        let i;
        for (i = 0; i < period - 1; ++i) {
            weight_sum += input[i] * (i + 1);
            sum += input[i];

            if (i >= period - period2) {
                weight_sum2 += input[i] * (i + 1 - (period - period2));
                sum2 += input[i];
            }
        }

        const buff = new ti_buffer(periodsqrt);

        for (i = period - 1; i < size; ++i) {
            weight_sum += input[i] * period;
            sum += input[i];

            weight_sum2 += input[i] * period2;
            sum2 += input[i];

            const wma = weight_sum / weights;
            const wma2 = weight_sum2 / weights2;
            const diff = 2 * wma2 - wma;

            weight_sumsqrt += diff * periodsqrt;
            sumsqrt += diff;

            buff.qpush(diff);

            if (i >= (period - 1) + (periodsqrt - 1)) {
                output.push(weight_sumsqrt / weightssqrt);

                weight_sumsqrt -= sumsqrt;
                // sumsqrt -= ti_buffer_get(buff, 1)
                sumsqrt -= buff.get(1);
            } else {
                weight_sumsqrt -= sumsqrt;
            }


            weight_sum -= sum;
            sum -= input[i - period + 1];

            weight_sum2 -= sum2;
            sum2 -= input[i - period2 + 1];
        }

        return output
    }

    var hma_1 = hma;

    /**
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function kama(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        const short_per = 2 / (2.0 + 1);
        const long_per = 2 / (30.0 + 1);

        let sum = 0;

        for (let i = 1; i < period; ++i) {
            sum += Math.abs(source[i] - source[i - 1]);
        }

        let kama = source[period - 1];
        output.push(kama);
        let er;
        let sc;

        for (let i = period; i < size; ++i) {
            sum += Math.abs(source[i] - source[i - 1]);

            if (i > period) {
                sum -= Math.abs(source[i - period] - source[i - period - 1]);
            }

            if (sum != 0.0) {
                er = Math.abs(source[i] - source[i - period]) / sum;
            } else {
                er = 1.0;
            }
            sc = Math.pow(er * (short_per - long_per) + long_per, 2);

            kama = kama + sc * (source[i] - kama);
            output.push(kama);
        }

        return output
    }

    var kama_1 = kama;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} period 
     * @param {number} multiple 
     * @param {number} [size]
     * @returns [kc_lower, kc_middle, kc_upper]
     */
    async function kc(
        high, low,
        close, period,
        multiple, size = high.length
    ) {

        const kc_lower = [];
        const kc_middle = [];
        const kc_upper = [];

        // if (period < 1) throw new Error("Invalid Options")

        const per = 2 / (period + 1);

        let price_ema = close[0];
        let tr_ema = high[0] - low[0];

        kc_lower.push(price_ema - multiple * tr_ema);
        kc_middle.push(price_ema);
        kc_upper.push(price_ema + multiple * tr_ema);

        let truerange = 0;
        for (let i = 1; i < size; ++i) {
            price_ema = (close[i] - price_ema) * per + price_ema;

            // Start CALC_TRUERANGE
            const l = low[i];
            const h = high[i];
            const c = close[i - 1];

            const ych = Math.abs(h - c);
            const ycl = Math.abs(l - c);

            let v = h - l;

            if (ych > v) v = ych;
            if (ycl > v) v = ycl;
            truerange = v;
            // End CALC_TRUERANGE

            tr_ema = (truerange - tr_ema) * per + tr_ema;

            kc_lower.push(price_ema - multiple * tr_ema);
            kc_middle.push(price_ema);
            kc_upper.push(price_ema + multiple * tr_ema);
        }

        return [kc_lower, kc_middle, kc_upper]
    }

    var kc_1 = kc;

    /**
     * 
     * @param {number[]} source 
     * @param {number} roc1 
     * @param {number} roc2 
     * @param {number} roc3 
     * @param {number} roc4 
     * @param {number} ma1 
     * @param {number} ma2 
     * @param {number} ma3 
     * @param {number} ma4 
     * @param {number} [size]
     * @returns 
     */
    async function kst(
        source,
        roc1, roc2, roc3, roc4,
        ma1, ma2, ma3, ma4,
        size = source.length
    ) {

        const kst = [];
        const kst_signal = [];
        // End MAX4

        const per1 = 2 / (ma1 + 1);
        const per2 = 2 / (ma2 + 1);
        const per3 = 2 / (ma3 + 1);
        const per4 = 2 / (ma4 + 1);
        const per_signal = 2 / (9 + 1);

        /**
         * 
         * @param {number} idx 
         * @param {number} period 
         * @returns 
         */
        function ROC(idx, period) {
            return ((source[idx] - source[idx - period]) / source[idx - period])
        }

        let _1 = ROC(roc1, roc1);
        let _2 = ROC(roc2, roc2);
        let _3 = ROC(roc3, roc3);
        let _4 = ROC(roc4, roc4);

        for (let i = roc1 + 1; i < roc4 + 1 && i < size; ++i) {
            _1 = (ROC(i, roc1) - _1) * per1 + _1;
        }
        for (let i = roc2 + 1; i < roc4 + 1 && i < size; ++i) {
            _2 = (ROC(i, roc2) - _2) * per2 + _2;
        }
        for (let i = roc3 + 1; i < roc4 + 1 && i < size; ++i) {
            _3 = (ROC(i, roc3) - _3) * per3 + _3;
        }
        for (let i = roc4 + 1; i < roc4 + 1 && i < size; ++i) {
            _4 = (ROC(i, roc4) - _4) * per4 + _4;
        }

        let val = (_1 * 1 + _2 * 2 + _3 * 3 + _4 * 4) / 10;
        kst_signal.push(val);

        let _signal = val;
        kst.push(_signal);

        for (let i = roc4 + 1; i < size; ++i) {
            _1 = (ROC(i, roc1) - _1) * per1 + _1;
            _2 = (ROC(i, roc2) - _2) * per2 + _2;
            _3 = (ROC(i, roc3) - _3) * per3 + _3;
            _4 = (ROC(i, roc4) - _4) * per4 + _4;

            val = (_1 * 1 + _2 * 2 + _3 * 3 + _4 * 4) / 10;
            kst.push(val);

            _signal = (val - _signal) * per_signal + _signal;
            kst_signal.push(_signal);
        }

        return [kst, kst_signal]
    }

    var kst_1 = kst;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number[]} volume 
     * @param {number} short_period 
     * @param {number} long_period 
     * @param {number} [size]
     * @returns 
     */
    async function kvo(
        high, low,
        close, volume,
        short_period, long_period,
        size = high.length
    ) {

        // if (short_period < 1) throw new Error("Invalid Options")
        // if (long_period < short_period) throw new Error("Invalid Options")
        // if (size <= 1) throw new Error("Out of range")

        const short_per = 2 / (short_period + 1);
        const long_per = 2 / (long_period + 1);

        const output = [];
        let cm = 0;
        let prev_hlc = high[0] + low[0] + close[0];
        let trend = -1;

        let short_ema = 0;
        let long_ema = 0;

        for (let i = 1; i < size; ++i) {
            const hlc = high[i] + low[i] + close[i];
            const dm = high[i] - low[i];

            if (hlc > prev_hlc && trend != 1) {
                trend = 1;
                cm = high[i - 1] - low[i - 1];
            } else if (hlc < prev_hlc && trend != 0) {
                trend = 0;
                cm = high[i - 1] - low[i - 1];
            }

            cm += dm;

            const vf = volume[i] * Math.abs(dm / cm * 2 - 1) * 100 * (trend ? 1.0 : -1.0);

            if (i == 1) {
                short_ema = vf;
                long_ema = vf;
            } else {
                short_ema = (vf - short_ema) * short_per + short_ema;
                long_ema = (vf - long_ema) * long_per + long_ema;
            }

            output.push(short_ema - long_ema);

            prev_hlc = hlc;
        }

        return output
    }

    var kvo_1 = kvo;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function lag(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 0) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        for (let i = period; i < size; ++i) {
            output.push(source[i - period]);
        }

        return output
    }

    var lag_1 = lag;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function linreg(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        // Start LINEAR_REGRESSION
        // LINEAR_REGRESSION(size, input, period, output, period)
        let x = 0;
        let x2 = 0;

        let y = 0;
        let xy = 0;
        // Start INIT()
        const p = (1.0 / (period));
        // End INIT()
        for (let i = 0; i < (period) - 1; ++i) {
            x += i + 1;
            x2 += (i + 1) * (i + 1);
            xy += (source)[i] * (i + 1);
            y += (source)[i];
        }

        x += (period);
        x2 += (period) * (period);

        const bd = 1.0 / ((period) * x2 - x * x);

        for (let i = (period) - 1; i < (size); ++i) {
            xy += (source)[i] * (period);
            y += (source)[i];

            const b = ((period) * xy - x * y) * bd;
            // Start FINAL
            // ## forecast = period
            const a = (y - b * x) * p;
            output.push(a + b * (period));
            // Start FINAL

            xy -= y;
            y -= (source)[i - (period) + 1];
        }
        // End LINEAR_REGRESSION

        return output
    }

    var linreg_1 = linreg;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function linregintercept(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        // Start LINEAR_REGRESSION
        // LINEAR_REGRESSION(size, input, period, output, 1)
        let x = 0;
        let x2 = 0;

        let y = 0;
        let xy = 0;
        // Start INIT()
        const p = (1.0 / (period));
        // End INIT()
        for (let i = 0; i < (period) - 1; ++i) {
            x += i + 1;
            x2 += (i + 1) * (i + 1);
            xy += (source)[i] * (i + 1);
            y += (source)[i];
        }

        x += (period);
        x2 += (period) * (period);

        const bd = 1.0 / ((period) * x2 - x * x);

        for (let i = (period) - 1; i < (size); ++i) {
            xy += (source)[i] * (period);
            y += (source)[i];

            const b = ((period) * xy - x * y) * bd;
            // Start FINAL
            // ## forecast = 1
            const a = (y - b * x) * p;
            output.push(a + b * (1));
            // Start FINAL

            xy -= y;
            y -= (source)[i - (period) + 1];
        }
        // End LINEAR_REGRESSION

        return output
    }

    var linregintercept_1 = linregintercept;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function linregslope(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        // Start LINEAR_REGRESSION
        // LINEAR_REGRESSION(size, input, period, output, period)
        let x = 0;
        let x2 = 0;

        let y = 0;
        let xy = 0;
        for (let i = 0; i < (period) - 1; ++i) {
            x += i + 1;
            x2 += (i + 1) * (i + 1);
            xy += (source)[i] * (i + 1);
            y += (source)[i];
        }

        x += (period);
        x2 += (period) * (period);

        const bd = 1.0 / ((period) * x2 - x * x);

        for (let i = (period) - 1; i < (size); ++i) {
            xy += (source)[i] * (period);
            y += (source)[i];

            const b = ((period) * xy - x * y) * bd;
            // Start FINAL
            // ## forecast = 1
            output.push(b);
            // Start FINAL

            xy -= y;
            y -= (source)[i - (period) + 1];
        }
        // End LINEAR_REGRESSION

        return output
    }

    var linregslope_1 = linregslope;

    /**
     * 
     * @param {number[]} source 
     * @param {number} short_period 
     * @param {number} long_period 
     * @param {number} signal_period
     * @param {number} [size]
     * @returns [macd, signal, hist]
     */
     async function macd(
        source, short_period,
        long_period, signal_period,
        size = source.length
    ) {

        const macd = [];
        const signal = [];
        const hist = [];

        // if (short_period < 1) throw new Error("Invalid Options")
        // if (long_period < 2) throw new Error("Invalid Options")
        // if (long_period < short_period) throw new Error("Invalid Options")
        // if (signal_period < 1) throw new Error("Invalid Options")
        // if (size <= long_period - 1) throw new Error("Out of range")

        const short_per = 2 / (short_period + 1);
        const long_per = 2 / (long_period + 1);
        const signal_per = 2 / (signal_period + 1);

        // if (short_period == 12 && long_period == 26) {
        // 	short_per = 0.15
        // 	long_per = 0.075
        // }

        let short_ema = source[0];
        let long_ema = source[0];
        let signal_ema = 0;

        for (let i = 1; i < size; ++i) {
            short_ema = (source[i] - short_ema) * short_per + short_ema;
            long_ema = (source[i] - long_ema) * long_per + long_ema;
            const out = short_ema - long_ema;

            if (i == long_period - 1) {
                signal_ema = out;
            }
            if (i >= long_period - 1) {
                signal_ema = (out - signal_ema) * signal_per + signal_ema;

                macd.push(out);
                signal.push(signal_ema);
                hist.push(out - signal_ema);
            }
        }

        return [macd, signal, hist]
    }

    var macd_1 = macd;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} volume 
     * @param {number} [size]
     * @returns 
     */
     async function marketfi(
        high, low,
        volume, size = high.length
    ) {

        const output = [];

        // if (size <= 0) throw new Error("Out of range")

        for (let i = 0; i < size; ++i) {
            output.push((high[i] - low[i]) / volume[i]);
        }

        return output
    }

    var marketfi_1 = marketfi;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function mass(
        high, low,
        period, size = high.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= 16 + sum_p) throw new Error("Out of range")

        const per = 2 / (9.0 + 1);
        const per1 = 1.0 - per;

        let ema = high[0] - low[0];

        let ema2 = ema;

        const sum = {
            index: 0,
            pushes: 0,
            size: period,
            sum: 0,
            vals: []
        };

        for (let i = 0; i < size; ++i) {
            const hl = high[i] - low[i];

            ema = ema * per1 + hl * per;

            if (i == 8) {
                ema2 = ema;
            }

            if (i >= 8) {
                ema2 = ema2 * per1 + ema * per;

                if (i >= 16) {
                    // Start ti_buffer_push
                    // BUFFER = sum
                    // VAL    = ema/ema2
                    if (sum.pushes >= sum.size) {
                        sum.sum -= sum.vals[sum.index];
                    }

                    sum.sum += ema / ema2;
                    sum.vals[sum.index] = ema / ema2;
                    sum.pushes += 1;
                    sum.index = (sum.index + 1);
                    if (sum.index >= sum.size) sum.index = 0;
                    // End ti_buffer_push

                    if (i >= 16 + period - 1) {
                        output.push(sum.sum);
                    }
                }
            }
        }

        return output
    }

    var mass_1 = mass;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function max(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let trail = 0;
        let maxi = -1;
        let max = source[0];

        let j;
        for (let i = period - 1; i < size; ++i, ++trail) {
            let bar = source[i];

            if (maxi < trail) {
                maxi = trail;
                max = source[maxi];
                j = trail;

                while (++j <= i) {
                    bar = source[j];

                    if (bar >= max) {
                        max = bar;
                        maxi = j;
                    }
                }

            } else if (bar >= max) {
                maxi = i;
                max = bar;
            }

            output.push(max);
        }

        return output
    }

    var max_1 = max;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function md(
        source, period,
        size = source.length
    ) {

        const output = [];

        const scale = 1.0 / period;

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let sum = 0;

        let j;
        for (let i = 0; i < size; ++i) {
            const today = source[i];
            sum += today;
            if (i >= period) sum -= source[i - period];

            const avg = sum * scale;

            if (i >= period - 1) {
                let acc = 0;
                for (j = 0; j < period; ++j) {
                    acc += Math.abs(avg - source[i - j]);
                }

                output.push(acc * scale);
            }
        }

        return output
    }

    var md_1 = md;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} [size]
     * @returns 
     */
     async function medprice(
        high, low,
        size = high.length
    ) {

        const output = [];

        for (let i = 0; i < size; ++i) {
            output.push((high[i] + low[i]) * 0.5);
        }

        return output
    }

    var medprice_1 = medprice;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number[]} volume 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function mfi(
        high, low,
        close, volume,
        period, size = high.length
    ) {

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        const output = [];

        // Start TYPPRICE()
        let ytyp = ((high[(0)] + low[(0)] + close[(0)]) * (1.0 / 3.0));
        // End TYPPRICE()

        const up = {
            size: period,
            index: 0,
            pushes: 0,
            sum: 0,
            vals: []
        };

        const down = {
            size: period,
            index: 0,
            pushes: 0,
            sum: 0,
            vals: []
        };

        for (let i = 1; i < size; ++i) {

            // Start TYPPRICE()
            const typ = ((high[(i)] + low[(i)] + close[(i)]) * (1.0 / 3.0));
            // End TYPPRICE()

            const bar = typ * volume[i];

            if (typ > ytyp) {
                // Start ti_buffer_push
                // BUFFER = up
                // VAL    = bar
                // ti_buffer_push(up, bar)
                if (up.pushes >= up.size) {
                    up.sum -= up.vals[up.index];
                }

                up.sum += bar;
                up.vals[up.index] = bar;
                up.pushes += 1;
                up.index = (up.index + 1);
                if (up.index >= up.size) up.index = 0;
                // End ti_buffer_push

                // Start ti_buffer_push
                // BUFFER = down
                // VAL    = 0.0
                // ti_buffer_push(down, 0.0)
                if (down.pushes >= down.size) {
                    down.sum -= down.vals[down.index];
                }

                down.sum += 0.0;
                down.vals[down.index] = 0.0;
                down.pushes += 1;
                down.index = (down.index + 1);
                if (down.index >= down.size) down.index = 0;
                // End ti_buffer_push
            } else if (typ < ytyp) {
                // Start ti_buffer_push
                // BUFFER = down
                // VAL    = bar
                // ti_buffer_push(down, bar)
                if (down.pushes >= down.size) {
                    down.sum -= down.vals[down.index];
                }

                down.sum += bar;
                down.vals[down.index] = bar;
                down.pushes += 1;
                down.index = (down.index + 1);
                if (down.index >= down.size) down.index = 0;
                // End ti_buffer_push

                // Start ti_buffer_push
                // BUFFER = up
                // VAL    = 0.0
                // ti_buffer_push(up, 0.0)
                if (up.pushes >= up.size) {
                    up.sum -= up.vals[up.index];
                }

                up.sum += 0.0;
                up.vals[up.index] = 0.0;
                up.pushes += 1;
                up.index = (up.index + 1);
                if (up.index >= up.size) up.index = 0;
                // End ti_buffer_push
            } else {
                // Start ti_buffer_push
                // BUFFER = up
                // VAL    = 0.0
                // ti_buffer_push(up, 0.0)
                if (up.pushes >= up.size) {
                    up.sum -= up.vals[up.index];
                }

                up.sum += 0.0;
                up.vals[up.index] = 0.0;
                up.pushes += 1;
                up.index = (up.index + 1);
                if (up.index >= up.size) up.index = 0;
                // End ti_buffer_push

                // Start ti_buffer_push
                // BUFFER = down
                // VAL    = 0.0
                // ti_buffer_push(down, 0.0)
                if (down.pushes >= down.size) {
                    down.sum -= down.vals[down.index];
                }

                down.sum += 0.0;
                down.vals[down.index] = 0.0;
                down.pushes += 1;
                down.index = (down.index + 1);
                if (down.index >= down.size) down.index = 0;
                // End ti_buffer_push
            }

            ytyp = typ;

            if (i >= period) {
                output.push((up.sum / (up.sum + down.sum) * 100.0));
            }
        }

        return output
    }

    var mfi_1 = mfi;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function min(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let trail = 0;
        let mini = -1;
        let min = source[0];

        let j;
        for (let i = period - 1; i < size; ++i, ++trail) {
            let bar = source[i];

            if (mini < trail) {
                mini = trail;
                min = source[mini];
                j = trail;

                while (++j <= i) {
                    bar = source[j];

                    if (bar <= min) {
                        min = bar;
                        mini = j;
                    }
                }

            } else if (bar <= min) {
                mini = i;
                min = bar;
            }

            output.push(min);
        }

        return output
    }

    var min_1 = min;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function mom(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        for (let i = period; i < size; ++i) {
            output.push(source[i] - source[i - period]);
        }

        return output
    }

    var mom_1 = mom;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function msw(
        source, period,
        size = source.length
    ) {

        const sine = [];
        const lead = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        const pi = 3.1415926;
        const tpi = 2 * pi;

        let weight = 0;
        let phase;
        let rp;
        let ip;

        let j;
        for (let i = period; i < size; ++i) {
            rp = 0;
            ip = 0;

            for (j = 0; j < period; ++j) {
                weight = source[i - j];
                rp = rp + Math.cos(tpi * j / period) * weight;
                ip = ip + Math.sin(tpi * j / period) * weight;
            }

            if (Math.abs(rp) > .001) {
                phase = Math.atan(ip / rp);
            } else {
                phase = tpi / 2.0 * (ip < 0 ? -1.0 : 1.0);
            }

            if (rp < 0.0) phase += pi;
            phase += pi / 2.0;

            if (phase < 0.0) phase += tpi;
            if (phase > tpi) phase -= tpi;

            sine.push(Math.sin(phase));
            lead.push(Math.sin(phase + pi / 4.0));
        }

        return [sine, lead]
    }

    var msw_1 = msw;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function natr(
        high, low,
        close, period,
        size = high.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        const per = 1.0 / (period);

        let sum = 0;
        let truerange;

        sum += high[0] - low[0];

        for (let i = 1; i < period; ++i) {
            // Start CALC_TRUERANGE()
            const l = low[i];
            const h = high[i];
            const c = close[i - 1];

            const ych = Math.abs(h - c);
            const ycl = Math.abs(l - c);

            let v = h - l;

            if (ych > v) v = ych;
            if (ycl > v) v = ycl;
            truerange = v;
            // End CALC_TRUERANGE()

            sum += truerange;
        }


        let val = sum / period;
        output.push(100 * (val) / close[period - 1]);

        for (let i = period; i < size; ++i) {
            // Start CALC_TRUERANGE()
            const l = low[i];
            const h = high[i];
            const c = close[i - 1];

            const ych = Math.abs(h - c);
            const ycl = Math.abs(l - c);

            let v = h - l;

            if (ych > v) v = ych;
            if (ycl > v) v = ycl;
            truerange = v;
            // End CALC_TRUERANGE()
            val = (truerange - val) * per + val;
            output.push(100 * (val) / close[i]);
        }

        return output
    }

    var natr_1 = natr;

    /**
     * 
     * @param {number} originalLength - example 100
     * @param {number[]} source - example ema | sma | rsi | etc.
     * @param {*} empty - example NaN | Null | 0 | false | etc.
     * @returns 
     */
     async function normalize(originalLength, source, empty = NaN) {
        const diff = originalLength - source.length;

        const emptyList = [];
        for (let index = 0; index < diff; ++index) {
            emptyList.push(empty);
        }

        const result = [...emptyList, ...source];

        return result
    }

    var normalize_1 = normalize;

    /**
     * More efficient
     * @param {number[]} source example: sma | rsi | cci | etc.
     * @param {number} length main source length, example: close.length
     * @returns 
     */
     async function normalize2(source, length) {
        const temp = [];
        const diff = length - source.length;
        temp[diff - 1] = NaN;
        return [...temp, ...source]
    }

    var normalize2_1 = normalize2;

    /**
     * 
     * @param {number[]} close 
     * @param {number[]} volume 
     * @param {number} [size]
     * @returns 
     */
     async function nvi(
        close, volume,
        size = close.length
    ) {

        const output = [];

        // if (size <= 0) throw new Error("Out of range")

        let nvi = 1000;

        output.push(nvi);

        for (let i = 1; i < size; ++i) {

            if (volume[i] < volume[i - 1]) {
                nvi += ((close[i] - close[i - 1]) / close[i - 1]) * nvi;
            }
            output.push(nvi);
        }

        return output
    }

    var nvi_1 = nvi;

    /**
     * 
     * @param {number[]} close 
     * @param {number[]} volume 
     * @param {number} [size]
     * @returns 
     */
     async function obv(
        close, volume,
        size = close.length
    ) {

        const output = [];
        let sum = 0;
        output.push(sum);

        let prev = close[0];

        for (let i = 1; i < size; ++i) {
            if (close[i] > prev) {
                sum += volume[i];
            } else if (close[i] < prev) {
                sum -= volume[i];
            } else ;

            prev = close[i];
            output.push(sum);
        }

        return output
    }

    var obv_1 = obv;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} period 
     * @param {number} [size]
     * @returns [pbands_lower, pbands_upper]
     */
     async function pbands(
        high, low,
        close, period,
        size = high.length
    ) {

        const pbands_lower = [];
        const pbands_upper = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let y_sum = 0.;
        let xy_sum = 0.;

        const x_sum = period * (period + 1) / 2.;
        const xsq_sum = period * (period + 1) * (2 * period + 1) / 6.;

        let i;
        for (i = 0; i < period; ++i) {
            xy_sum += close[i] * (i + 1);
            y_sum += close[i];
        }

        --i;

        const b = (xy_sum / period - x_sum / period * y_sum / period) / (xsq_sum / period - (x_sum / period) * (x_sum / period));

        let the_max = high[i];

        for (let j = 1; j < period; ++j) {
            if (the_max < high[i - j] + j * b) {
                the_max = high[i - j] + j * b;
            }
        }

        let the_min = low[i];
        for (let j = 1; j < period; ++j) {
            if (the_min > low[i - j] + j * b) {
                the_min = low[i - j] + j * b;
            }
        }

        pbands_upper.push(the_max);
        pbands_lower.push(the_min);

        ++i;

        for (; i < size; ++i) {
            xy_sum += -y_sum + close[i] * period;
            y_sum += -close[i - period] + close[i];

            const b = (xy_sum / period - x_sum / period * y_sum / period) / (xsq_sum / period - (x_sum / period) * (x_sum / period));

            let the_max = high[i];
            for (let j = 1; j < period; ++j) {
                if (the_max < high[i - j] + j * b) {
                    the_max = high[i - j] + j * b;
                }
            }

            let the_min = low[i];
            for (let j = 1; j < period; ++j) {
                if (the_min > low[i - j] + j * b) {
                    the_min = low[i - j] + j * b;
                }
            }

            pbands_lower.push(the_min);
            pbands_upper.push(the_max);
        }

        return [pbands_lower, pbands_upper]
    }

    var pbands_1 = pbands;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} ema_period 
     * @param {number} [size]
     * @returns 
     */
     async function pfe(
        source, period,
        ema_period, size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        const per = 2 / (ema_period + 1);

        const denom = {
            size: period,
            index: 0,
            pushes: 0,
            sum: 0,
            vals: []
        };

        let i;
        for (i = 1; i < period; ++i) {
            // Start ti_buffer_push
            // BUFFER = denom
            // VAL    = Math.sqrt(Math.pow(source[i] - source[i-1], 2) + 1.)
            // ti_buffer_push(denom, sqrt(pow(real[i] - real[i-1], 2) + 1.))
            if (denom.pushes >= denom.size) {
                denom.sum -= denom.vals[denom.index];
            }

            denom.sum += Math.sqrt(Math.pow(source[i] - source[i - 1], 2) + 1);
            denom.vals[denom.index] = Math.sqrt(Math.pow(source[i] - source[i - 1], 2) + 1);
            denom.pushes += 1;
            denom.index = (denom.index + 1);
            if (denom.index >= denom.size) denom.index = 0;
            // End ti_buffer_push
        }

        // Start ti_buffer_push
        // BUFFER = denom
        // VAL    = Math.sqrt(Math.pow(source[i] - source[i-1], 2) + 1.)
        // ti_buffer_push(denom, sqrt(pow(real[i] - real[i-1], 2) + 1.))
        if (denom.pushes >= denom.size) {
            denom.sum -= denom.vals[denom.index];
        }

        denom.sum += Math.sqrt(Math.pow(source[i] - source[i - 1], 2) + 1);
        denom.vals[denom.index] = Math.sqrt(Math.pow(source[i] - source[i - 1], 2) + 1);
        denom.pushes += 1;
        denom.index = (denom.index + 1);
        if (denom.index >= denom.size) denom.index = 0;
        // End ti_buffer_push

        // Start SIGN
        // let numer = SIGN(source[i] - source[i-period]) * 100. * Math.sqrt(Math.pow(source[i] - source[i-period], 2) + 100.)
        const numer = ((source[i] - source[i - period]) > 0 ? 1 : -1) * 100 * Math.sqrt(Math.pow(source[i] - source[i - period], 2) + 100);
        // End SIGN

        let ema = numer / denom.sum;
        output.push(ema);

        for (i = period + 1; i < size; ++i) {
            // Start ti_buffer_push
            // BUFFER = denom
            // VAL    = Math.sqrt(Math.pow(source[i] - source[i-1], 2) + 1.)
            // ti_buffer_push(denom, sqrt(pow(real[i] - real[i-1], 2) + 1.))
            if (denom.pushes >= denom.size) {
                denom.sum -= denom.vals[denom.index];
            }

            denom.sum += Math.sqrt(Math.pow(source[i] - source[i - 1], 2) + 1);
            denom.vals[denom.index] = Math.sqrt(Math.pow(source[i] - source[i - 1], 2) + 1);
            denom.pushes += 1;
            denom.index = (denom.index + 1);
            if (denom.index >= denom.size) denom.index = 0;
            // End ti_buffer_push

            // Start SIGN
            // let numer2 = SIGN(real[i] - real[i-(int)period]) * 100. * sqrt(pow(real[i] - real[i-(int)period], 2) + 100.)
            const numer2 = ((source[i] - source[i - period]) > 0 ? 1 : -1) * 100 * Math.sqrt(Math.pow(source[i] - source[i - period], 2) + 100);
            // End SIGN

            // Start EMA_NEXT
            // ema = EMA_NEXT(numer2 / denom->sum)
            ema = (((numer2 / denom.sum) - ema) * per + ema);
            // End EMA_NEXT

            output.push(ema);
        }

        return output
    }

    var pfe_1 = pfe;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} period 
     * @param {number} ema_period 
     * @param {number} [size]
     * @returns 
     */
     async function posc(
        high, low,
        close, period,
        ema_period, size = high.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (ema_period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let y_sum = 0;
        let xy_sum = 0;
        let ema;

        const x_sum = period * (period + 1) / 2;
        const xsq_sum = period * (period + 1) * (2 * period + 1) / 6;

        let i;
        for (i = 0; i < period; ++i) {
            xy_sum += close[i] * (i + 1);
            y_sum += close[i];
        }

        --i;

        const b = (xy_sum / period - x_sum / period * y_sum / period) / (xsq_sum / period - (x_sum / period) * (x_sum / period));

        let the_max = high[i];
        for (let j = 1; j < period; ++j) {
            if (the_max < high[i - j] + j * b) {
                the_max = high[i - j] + j * b;
            }
        }
        let the_min = low[i];
        for (let j = 1; j < period; ++j) {
            if (the_min > low[i - j] + j * b) {
                the_min = low[i - j] + j * b;
            }
        }

        ema = (close[i] - the_min) / (the_max - the_min) * 100;
        output.push(ema);

        ++i;

        for (; i < size; ++i) {
            xy_sum += -y_sum + close[i] * period;
            y_sum += -close[i - period] + close[i];

            const b = (xy_sum / period - x_sum / period * y_sum / period) / (xsq_sum / period - (x_sum / period) * (x_sum / period));

            let the_max = high[i];
            for (let j = 1; j < period; ++j) {
                if (the_max < high[i - j] + j * b) {
                    the_max = high[i - j] + j * b;
                }
            }

            let the_min = low[i];
            for (let j = 1; j < period; ++j) {
                if (the_min > low[i - j] + j * b) {
                    the_min = low[i - j] + j * b;
                }
            }

            const osc = (close[i] - the_min) / (the_max - the_min) * 100;
            ema = (osc - ema) * 2. / (1 + ema_period) + ema;
            output.push(ema);
        }

        return output
    }

    var posc_1 = posc;

    /**
     * 
     * @param {number[]} source 
     * @param {number} short_period 
     * @param {number} long_period 
     * @param {number} [size]
     * @returns 
     */
     async function ppo(
        source, short_period,
        long_period, size = source.length
    ) {

        const ppo = [];

        // if (short_period < 1) throw new Error("Invalid Options")
        // if (long_period < 2) throw new Error("Invalid Options")
        // if (long_period < short_period) throw new Error("Invalid Options")
        // if (size <= 1) throw new Error("Out of range")

        const short_per = 2 / (short_period + 1);
        const long_per = 2 / (long_period + 1);

        let short_ema = source[0];
        let long_ema = source[0];

        for (let i = 1; i < size; ++i) {
            short_ema = (source[i] - short_ema) * short_per + short_ema;
            long_ema = (source[i] - long_ema) * long_per + long_ema;
            const out = 100.0 * (short_ema - long_ema) / long_ema;

            ppo.push(out);
        }

        return ppo
    }

    var ppo_1 = ppo;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number} accel_step 
     * @param {number} accel_max 
     * @param {number} [size]
     * @returns 
     */
     async function psar(
        high, low,
        accel_step, accel_max,
        size = high.length
    ) {

        const output = [];

        // if (accel_step <= 0) throw new Error("Invalid Options")
        // if (accel_max <= accel_step) throw new Error("Invalid Options")
        // if (size < 2) throw new Error("Out of range")

        let lng;
        if (high[0] + low[0] <= high[1] + low[1]) {
            lng = 1;
        } else {
            lng = 0;
        }

        let sar;
        let extreme;

        if (lng) {
            extreme = high[0];
            sar = low[0];
        } else {
            extreme = low[0];
            sar = high[0];
        }

        let accel = accel_step;

        for (let i = 1; i < size; ++i) {

            sar = (extreme - sar) * accel + sar;

            if (lng) {

                if (i >= 2 && (sar > low[i - 2])) sar = low[i - 2];

                if ((sar > low[i - 1])) sar = low[i - 1];

                if (accel < accel_max && high[i] > extreme) {
                    accel += accel_step;
                    if (accel > accel_max) accel = accel_max;
                }

                if (high[i] > extreme) extreme = high[i];

            } else {

                if (i >= 2 && (sar < high[i - 2])) sar = high[i - 2];

                if ((sar < high[i - 1])) sar = high[i - 1];

                if (accel < accel_max && low[i] < extreme) {
                    accel += accel_step;
                    if (accel > accel_max) accel = accel_max;
                }

                if (low[i] < extreme) extreme = low[i];
            }

            if ((lng && low[i] < sar) || (!lng && high[i] > sar)) {
                accel = accel_step;
                sar = extreme;

                lng = !lng;

                if (!lng) {
                    extreme = low[i];
                } else {
                    extreme = high[i];
                }
            }

            output.push(sar);
        }

        return output
    }

    var psar_1 = psar;

    /**
     * 
     * @param {number[]} close 
     * @param {number[]} volume 
     * @param {number} [size]
     * @returns 
     */
     async function pvi(
        close, volume,
        size = close.length
    ) {

        const output = [];

        // if (size <= 0) throw new Error("Out of range")

        let pvi = 1000;
        output.push(pvi);

        for (let i = 1; i < size; ++i) {

            if (volume[i] > volume[i - 1]) {
                pvi += ((close[i] - close[i - 1]) / close[i - 1]) * pvi;
            }
            output.push(pvi);
        }

        return output
    }

    var pvi_1 = pvi;

    /**
     * 
     * @param {number[]} open 
     * @param {number[]} close 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function qstick(
        open,
        close,
        period,
        size = close.length
    ) {

        const output = [];
        const scale = 1.0 / period;

        let sum = 0;

        let i;
        for (i = 0; i < period; ++i) {
            sum += close[i] - open[i];
        }

        output.push(sum * scale);

        for (i = period; i < size; ++i) {
            sum += close[i] - open[i];
            sum -= close[i - period] - open[i - period];
            output.push(sum * scale);
        }

        return output
    }

    var qstick_1 = qstick;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} lookback_period 
     * @param {number} [size]
     * @returns 
     */
     async function rmi(
        source, period,
        lookback_period, size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (lookback_period < 1) throw new Error("Invalid Options")
        // if (size <= lookback_period) throw new Error("Out of range")

        let gains_ema;
        let losses_ema;

        let i = lookback_period;
        gains_ema = ((0) > (source[i] - source[i - lookback_period]) ? (0) : (source[i] - source[i - lookback_period]));
        losses_ema = ((0) > (source[i - lookback_period] - source[i]) ? (0) : (source[i - lookback_period] - source[i]));
        ++i;

        output.push(gains_ema / (gains_ema + losses_ema) * 100);

        for (; i < size; ++i) {
            gains_ema = ((0) > (source[i] - source[i - lookback_period]) ? (0) : (source[i] - source[i - lookback_period]) - gains_ema) * 2. / (1 + period) + gains_ema;
            losses_ema = ((0) > (source[i - lookback_period] - source[i]) ? (0) : (source[i - lookback_period] - source[i]) - losses_ema) * 2. / (1 + period) + losses_ema;
            output.push(gains_ema / (gains_ema + losses_ema) * 100);
        }

        return output
    }

    var rmi_1 = rmi;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} beta 
     * @param {number} [size]
     * @returns 
     */
     async function rmta(
        source, period,
        beta, size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        const alpha = 1 - beta;
        let b = (1 - alpha) * source[0] + source[0];
        let rmta = (1 - alpha) * source[0] + alpha * (source[0] + b);

        for (let i = 1; i < period - 1; ++i) {
            const next_b = (1 - alpha) * b + source[i];
            rmta = (1 - alpha) * rmta + alpha * (source[i] + next_b - b);
            b = next_b;
        }

        for (let i = period - 1; i < size; ++i) {
            const next_b = (1. - alpha) * b + source[i];
            rmta = (1 - alpha) * rmta + alpha * (source[i] + next_b - b);
            b = next_b;
            output.push(rmta);
        }

        return output
    }

    var rmta_1 = rmta;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function roc(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        for (let i = period; i < size; ++i) {
            output.push((source[i] - source[i - period]) / source[i - period]);
        }

        return output
    }

    var roc_1 = roc;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function rocr(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        for (let i = period; i < size; ++i) {
            output.push(source[i] / source[i - period]);
        }

        return output
    }

    var rocr_1 = rocr;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function rsi(
        source, period,
        size = source.length
    ) {

        const output = [];

        const per = 1.0 / (period);

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        let smooth_up = 0;
        let smooth_down = 0;

        for (let i = 1; i <= period; ++i) {
            const upward = source[i] > source[i - 1] ? source[i] - source[i - 1] : 0;
            const downward = source[i] < source[i - 1] ? source[i - 1] - source[i] : 0;
            smooth_up += upward;
            smooth_down += downward;
        }

        smooth_up /= period;
        smooth_down /= period;
        output.push(100.0 * (smooth_up / (smooth_up + smooth_down)));

        for (let i = period + 1; i < size; ++i) {
            const upward = (source[i] > source[i - 1] ? source[i] - source[i - 1] : 0);
            const downward = (source[i] < source[i - 1] ? source[i - 1] - source[i] : 0);

            smooth_up = (upward - smooth_up) * per + smooth_up;
            smooth_down = (downward - smooth_down) * per + smooth_down;

            output.push(100.0 * (smooth_up / (smooth_up + smooth_down)));
        }

        return output
    }

    var rsi_1 = rsi;

    /**
     * 
     * @param {number[]} source 
     * @param {number} sma_period 
     * @param {number} stddev_period 
     * @param {number} [size]
     * @returns 
     */
     async function rvi(
        source, sma_period,
        stddev_period, size = source.length
    ) {

        const output = [];

        // if (sma_period < 1) throw new Error("Invalid Options")
        // if (stddev_period < 1) throw new Error("Invalid Options")
        // if (size <= stddev_period-1) throw new Error("Out of range")

        let y_sum = 0;
        let xy_sum = 0;

        const x_sum = stddev_period * (stddev_period + 1) / 2;
        const xsq_sum = stddev_period * (stddev_period + 1) * (2 * stddev_period + 1) / 6;

        let gains_ema = 0;
        let losses_ema = 0;

        let i = 0;
        for (; i < stddev_period; ++i) {
            xy_sum += source[i] * (i + 1);
            y_sum += source[i];
        }

        --i;
        const b = (xy_sum / stddev_period - x_sum / stddev_period * y_sum / stddev_period) / (xsq_sum / stddev_period - (x_sum / stddev_period) * (x_sum / stddev_period));
        const a = y_sum / stddev_period - b * x_sum / stddev_period;

        const higher = source[i] - (a + b * stddev_period);
        if (higher > 0) {
            gains_ema = higher * higher / stddev_period;
        } else {
            losses_ema = higher * higher / stddev_period;
        }

        if (gains_ema + losses_ema == 0) {
            output.push(50);
        } else {
            output.push(gains_ema / (gains_ema + losses_ema) * 100);
        }
        ++i;

        for (; i < size; ++i) {
            xy_sum += -y_sum + source[i] * stddev_period;
            y_sum += -source[i - stddev_period] + source[i];

            const b = (xy_sum / stddev_period - x_sum / stddev_period * y_sum / stddev_period) / (xsq_sum / stddev_period - (x_sum / stddev_period) * (x_sum / stddev_period));
            const a = y_sum / stddev_period - b * x_sum / stddev_period;

            const higher = source[i] - (a + b * stddev_period);

            if (higher > 0) {
                gains_ema = (higher * higher / stddev_period - gains_ema) * 2. / (sma_period + 1) + gains_ema;
            } else {
                losses_ema = (higher * higher / stddev_period - losses_ema) * 2. / (sma_period + 1) + losses_ema;
            }

            if (gains_ema + losses_ema == 0) {
                output.push(50);
            } else {
                output.push(gains_ema / (gains_ema + losses_ema) * 100);
            }
        }

        return output
    }

    var rvi_1 = rvi;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function sma(
        source, period,
        size = source.length
    ) {

        const output = [];

        const scale = 1.0 / period;

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let sum = 0;

        for (let i = 0; i < period; ++i) {
            sum += source[i];
        }

        output.push(sum * scale);

        for (let i = period; i < size; ++i) {
            sum += source[i];
            sum -= source[i - period];
            output.push(sum * scale);
        }

        return output
    }

    var sma_1 = sma;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} q_period 
     * @param {number} r_period 
     * @param {number} s_period 
     * @param {number} [size]
     * @returns 
     */
     async function smi(
        high, low,
        close, q_period,
        r_period, s_period,
        size = high.length
    ) {

        const output = [];

        // if (q_period < 1 || r_period < 1 || s_period < 1) {
        //     throw new Error("Invalid Options"
        // }

        let progress = -q_period + 1;

        let ema_r_num = NaN;
        let ema_s_num = NaN;
        let ema_r_den = NaN;
        let ema_s_den = NaN;
        let ll = 0;
        let hh = 0;
        let hh_idx = 0;
        let ll_idx = 0;

        let var1 = 0;

        let i = 0;
        for (; i < size && progress == -q_period + 1; ++i, ++progress) {
            hh = high[i];
            hh_idx = progress;
            ll = low[i];
            ll_idx = progress;
        }

        for (; i < size && progress < 0; ++i, ++progress) {
            if (hh <= high[i]) {
                hh = high[i];
                hh_idx = progress;
            }

            if (ll >= low[i]) {
                ll = low[i];
                ll_idx = progress;
            }
        }

        for (; i < size && progress == 0; ++i, ++progress) {
            if (hh <= high[i]) {
                hh = high[i];
                hh_idx = progress;
            }

            if (ll >= low[i]) {
                ll = low[i];
                ll_idx = progress;
            }

            ema_r_num = ema_s_num = close[i] - 0.5 * (hh + ll);
            ema_r_den = ema_s_den = hh - ll;

            output.push(100 * ema_s_num / (0.5 * ema_s_den));
        }

        for (; i < size; ++i, ++progress) {
            if (hh_idx == progress - q_period) {
                hh = high[i];
                hh_idx = progress;
                for (let j = 1; j < q_period; ++j) {
                    var1 = high[i - j];
                    if (var1 > hh) {
                        hh = var1;
                        hh_idx = progress - j;
                    }
                }

            } else if (hh <= high[i]) {
                hh = high[i];
                hh_idx = progress;
            }

            if (ll_idx == progress - q_period) {
                ll = low[i];
                ll_idx = progress;
                for (let j = 1; j < q_period; ++j) {
                    var1 = low[i - j];
                    if (var1 < ll) {
                        ll = var1;
                        ll_idx = progress - j;
                    }
                }

            } else if (ll >= low[i]) {
                ll = low[i];
                ll_idx = progress;
            }

            ema_r_num = ((close[i] - 0.5 * (hh + ll)) - ema_r_num) * (2 / (1 + r_period)) + ema_r_num;
            ema_s_num = (ema_r_num - ema_s_num) * (2 / (1 + s_period)) + ema_s_num;

            ema_r_den = ((hh - ll) - ema_r_den) * (2 / (1 + r_period)) + ema_r_den;
            ema_s_den = (ema_r_den - ema_s_den) * (2 / (1 + s_period)) + ema_s_den;

            output.push(100 * ema_s_num / (0.5 * ema_s_den));
        }

        return output
    }

    var smi_1 = smi;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function stddev(
        source, period,
        size = source.length
    ) {

        const output = [];

        const scale = 1.0 / period;

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let sum = 0;
        let sum2 = 0;

        for (let i = 0; i < period; ++i) {
            sum += source[i];
            sum2 += source[i] * source[i];
        }

        let s2s2 = (sum2 * scale - (sum * scale) * (sum * scale));
        if (s2s2 > 0.0) s2s2 = Math.sqrt(s2s2);
        output.push(s2s2);

        for (let i = period; i < size; ++i) {
            sum += source[i];
            sum2 += source[i] * source[i];

            sum -= source[i - period];
            sum2 -= source[i - period] * source[i - period];

            let s2s2 = (sum2 * scale - (sum * scale) * (sum * scale));
            if (s2s2 > 0.0) s2s2 = Math.sqrt(s2s2);
            output.push(s2s2);
        }

        return output
    }

    var stddev_1 = stddev;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function stderr(
        source, period,
        size = source.length
    ) {

        const output = [];

        const scale = 1.0 / period;

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let sum = 0;
        let sum2 = 0;

        const mul = 1.0 / Math.sqrt(period);

        for (let i = 0; i < period; ++i) {
            sum += source[i];
            sum2 += source[i] * source[i];
        }

        let s2s2 = (sum2 * scale - (sum * scale) * (sum * scale));
        if (s2s2 > 0.0) s2s2 = Math.sqrt(s2s2);
        output.push(mul * s2s2);

        for (let i = period; i < size; ++i) {
            sum += source[i];
            sum2 += source[i] * source[i];

            sum -= source[i - period];
            sum2 -= source[i - period] * source[i - period];

            let s2s2 = (sum2 * scale - (sum * scale) * (sum * scale));
            if (s2s2 > 0.0) s2s2 = Math.sqrt(s2s2);
            output.push(mul * s2s2);
        }

        return output
    }

    var stderr_1 = stderr;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} kperiod 
     * @param {number} kslow 
     * @param {number} dperiod 
     * @param {number} [size]
     * @returns [stoch, stoch_ma]
     */
     async function stoch(
        high, low,
        close,
        kperiod, kslow, dperiod,
        size = high.length
    ) {

        const kper = 1.0 / kslow;
        const dper = 1.0 / dperiod;

        const stoch = [];
        const stoch_ma = [];

        // if (kperiod < 1) throw new Error("Invalid Options")
        // if (kslow < 1) throw new Error("Invalid Options")
        // if (dperiod < 1) throw new Error("Invalid Options")
        // if (size <= (kperiod + kslow + dperiod - 3)) throw new Error("Out of range")

        let trail = 0;
        let maxi = -1;
        let mini = -1;
        let max = high[0];
        let min = low[0];
        let bar;

        const k_sum = {
            size: kslow,
            index: 0,
            pushes: 0,
            sum: 0,
            vals: []
        };

        const d_sum = {
            size: dperiod,
            index: 0,
            pushes: 0,
            sum: 0,
            vals: []
        };

        let j;
        for (let i = 0; i < size; ++i) {

            if (i >= kperiod) ++trail;

            // Maintain highest
            bar = high[i];
            if (maxi < trail) {
                maxi = trail;
                max = high[maxi];
                j = trail;

                while (++j <= i) {
                    bar = high[j];
                    if (bar >= max) {
                        max = bar;
                        maxi = j;
                    }
                }

            } else if (bar >= max) {
                maxi = i;
                max = bar;
            }

            // Maintain lowest
            bar = low[i];
            if (mini < trail) {
                mini = trail;
                min = low[mini];
                j = trail;

                while (++j <= i) {
                    bar = low[j];
                    if (bar <= min) {
                        min = bar;
                        mini = j;
                    }
                }

            } else if (bar <= min) {
                mini = i;
                min = bar;
            }

            // Calculate it
            const kdiff = (max - min);
            const kfast = kdiff == 0.0 ? 0.0 : 100 * ((close[i] - min) / kdiff);

            // Start ti_buffer_push
            // BUFFER = k_sum
            // VAL    = kfast
            // ti_buffer_push(k_sum, kfast)
            if (k_sum.pushes >= k_sum.size) {
                k_sum.sum -= k_sum.vals[k_sum.index];
            }

            k_sum.sum += kfast;
            k_sum.vals[k_sum.index] = kfast;
            k_sum.pushes += 1;
            k_sum.index = (k_sum.index + 1);
            if (k_sum.index >= k_sum.size) k_sum.index = 0;
            // End ti_buffer_push


            if (i >= kperiod - 1 + kslow - 1) {
                const k = k_sum.sum * kper;
                // Start ti_buffer_push
                // BUFFER = d_sum
                // VAL    = k
                // ti_buffer_push(d_sum, k)
                if (d_sum.pushes >= d_sum.size) {
                    d_sum.sum -= d_sum.vals[d_sum.index];
                }

                d_sum.sum += k;
                d_sum.vals[d_sum.index] = k;
                d_sum.pushes += 1;
                d_sum.index = (d_sum.index + 1);
                if (d_sum.index >= d_sum.size) d_sum.index = 0;
                // End ti_buffer_push

                if (i >= kperiod - 1 + kslow - 1 + dperiod - 1) {
                    stoch.push(k);
                    stoch_ma.push(d_sum.sum * dper);
                }
            }
        }

        return [stoch, stoch_ma]
    }

    var stoch_1 = stoch;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function stochrsi(
        source, period,
        size = source.length
    ) {

        const output = [];

        const per = 1.0 / (period);

        // if (period < 2) throw new Error("Invalid Options")
        // if (size <= (period) * 2 - 1) throw new Error("Out of range")

        const rsi = {
            size: period,
            index: 0,
            pushes: 0,
            sum: 0,
            vals: []
        };

        let smooth_up = 0;
        let smooth_down = 0;

        for (let i = 1; i <= period; ++i) {
            const upward = source[i] > source[i - 1] ? source[i] - source[i - 1] : 0;
            const downward = source[i] < source[i - 1] ? source[i - 1] - source[i] : 0;
            smooth_up += upward;
            smooth_down += downward;
        }

        smooth_up /= period;
        smooth_down /= period;
        let r = 100.0 * (smooth_up / (smooth_up + smooth_down));

        // Start ti_buffer_push
        // BUFFER = rsi
        // VAL    = r
        // ti_buffer_push(rsi, r)
        if (rsi.pushes >= rsi.size) {
            rsi.sum -= rsi.vals[rsi.index];
        }

        rsi.sum += r;
        rsi.vals[rsi.index] = r;
        rsi.pushes += 1;
        rsi.index = (rsi.index + 1);
        if (rsi.index >= rsi.size) rsi.index = 0;
        // End ti_buffer_push

        let min = r;
        let max = r;
        let mini = 0;
        let maxi = 0;

        for (let i = period + 1; i < size; ++i) {
            const upward = source[i] > source[i - 1] ? source[i] - source[i - 1] : 0;
            const downward = source[i] < source[i - 1] ? source[i - 1] - source[i] : 0;

            smooth_up = (upward - smooth_up) * per + smooth_up;
            smooth_down = (downward - smooth_down) * per + smooth_down;

            r = 100.0 * (smooth_up / (smooth_up + smooth_down));

            if (r > max) {
                max = r;
                maxi = rsi.index;
            } else if (maxi == rsi.index) {
                max = r;
                for (let j = 0; j < rsi.size; ++j) {
                    if (j == rsi.index) continue

                    if (rsi.vals[j] > max) {
                        max = rsi.vals[j];
                        maxi = j;
                    }
                }
            }

            if (r < min) {
                min = r;
                mini = rsi.index;
            } else if (mini == rsi.index) {
                min = r;
                for (let j = 0; j < rsi.size; ++j) {
                    if (j == rsi.index) continue
                    if (rsi.vals[j] < min) {
                        min = rsi.vals[j];
                        mini = j;
                    }
                }
            }

            // Start ti_buffer_qpush
            // BUFFER = rsi
            // VAL    = r
            // ti_buffer_qpush(rsi, r)
            rsi.vals[rsi.index] = r;
            rsi.index = rsi.index + 1;
            if (rsi.index >= rsi.size) rsi.index = 0;
            // End ti_buffer_qpush

            if (i > period * 2 - 2) {
                const diff = max - min;
                if (diff == 0.0) {
                    output.push(0.0);
                } else {
                    output.push((r - min) / (diff));
                }
            }
        }

        return output
    }

    var stochrsi_1 = stochrsi;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function sum(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let sum = 0;

        for (let i = 0; i < period; ++i) {
            sum += source[i];
        }

        output.push(sum);

        for (let i = period; i < size; ++i) {
            sum += source[i];
            sum -= source[i - period];
            output.push(sum);
        }

        return output
    }

    var sum_1 = sum;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
     async function tema(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= (period - 1) * 3) throw new Error("Out of range")

        const per = 2 / (period + 1);
        const per1 = 1.0 - per;

        let ema = source[0];
        let ema2 = 0;
        let ema3 = 0;

        for (let i = 0; i < size; ++i) {
            ema = ema * per1 + source[i] * per;
            if (i == period - 1) {
                ema2 = ema;
            }

            if (i >= period - 1) {
                ema2 = ema2 * per1 + ema * per;
                if (i == (period - 1) * 2) {
                    ema3 = ema2;
                }

                if (i >= (period - 1) * 2) {
                    ema3 = ema3 * per1 + ema2 * per;

                    if (i >= (period - 1) * 3) {
                        output.push(3 * ema - 3 * ema2 + ema3);
                    }
                }
            }
        }

        return output
    }

    var tema_1 = tema;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} [size]
     * @returns 
     */
    async function tr(
        high, low,
        close, size = high.length
    ) {

        const output = [];
        let truerange;

        output[0] = high[0] - low[0];

        for (let i = 1; i < size; ++i) {
            // Start CALC_TRUERANGE
            const l = low[i];
            const h = high[i];
            const c = close[i - 1];

            const ych = Math.abs(h - c);
            const ycl = Math.abs(l - c);

            let v = h - l;

            if (ych > v) v = ych;
            if (ycl > v) v = ycl;
            truerange = v;
            // End CALC_TRUERANGE
            output.push(truerange);
        }

        return output
    }

    var tr_1 = tr;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function trima(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        const weights = 1 / ((period % 2) ?
            ((period / 2 + 1) * (period / 2 + 1)) :
            ((period / 2 + 1) * (period / 2)));

        let weight_sum = 0;
        let lead_sum = 0;
        let trail_sum = 0;

        const lead_period = period % 2 ? period / 2 : period / 2 - 1;
        const trail_period = lead_period + 1;

        let w = 1;

        for (let i = 0; i < period - 1; ++i) {
            weight_sum += source[i] * w;

            if (i + 1 > period - lead_period) lead_sum += source[i];
            if (i + 1 <= trail_period) trail_sum += source[i];

            if (i + 1 < trail_period) ++w;
            if (i + 1 >= period - lead_period) --w;
        }


        let lsi = (period - 1) - lead_period + 1;
        let tsi1 = (period - 1) - period + 1 + trail_period;
        let tsi2 = (period - 1) - period + 1;

        for (let i = period - 1; i < size; ++i) {
            weight_sum += source[i];
            output.push(weight_sum * weights);

            lead_sum += source[i];

            weight_sum += lead_sum;
            weight_sum -= trail_sum;

            lead_sum -= source[lsi++];
            trail_sum += source[tsi1++];
            trail_sum -= source[tsi2++];
        }

        return output
    }

    var trima_1 = trima;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function trix(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= ((period-1)*3)+1) throw new Error("Out of range")

        const start = (period * 3) - 2;
        const per = 2 / (period + 1);

        let ema1 = source[0];
        let ema2 = 0;
        let ema3 = 0;

        for (let i = 1; i < start; ++i) {
            ema1 = (source[i] - ema1) * per + ema1;

            if (i == period - 1) {
                ema2 = ema1;
            } else if (i > period - 1) {
                ema2 = (ema1 - ema2) * per + ema2;

                if (i == period * 2 - 2) {
                    ema3 = ema2;
                } else if (i > period * 2 - 2) {
                    ema3 = (ema2 - ema3) * per + ema3;
                }
            }
        }

        for (let i = start; i < size; ++i) {
            ema1 = (source[i] - ema1) * per + ema1;
            ema2 = (ema1 - ema2) * per + ema2;
            const last = ema3;
            ema3 = (ema2 - ema3) * per + ema3;
            output.push((ema3 - last) / ema3 * 100.0);
        }

        return output
    }

    var trix_1 = trix;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function tsf(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        // Start LINEAR_REGRESSION
        // LINEAR_REGRESSION(size, input, period, output, period + 1)
        let x = 0;
        let x2 = 0;

        let y = 0;
        let xy = 0;
        // Start INIT()
        const p = (1.0 / (period));
        // End INIT()
        for (let i = 0; i < (period) - 1; ++i) {
            x += i + 1;
            x2 += (i + 1) * (i + 1);
            xy += (source)[i] * (i + 1);
            y += (source)[i];
        }

        x += (period);
        x2 += (period) * (period);

        const bd = 1.0 / ((period) * x2 - x * x);

        for (let i = (period) - 1; i < (size); ++i) {
            xy += (source)[i] * (period);
            y += (source)[i];

            const b = ((period) * xy - x * y) * bd;
            // Start FINAL
            // ## forecast = period + 1
            const a = (y - b * x) * p;
            output.push(a + b * (period + 1));
            // Start FINAL

            xy -= y;
            y -= (source)[i - (period) + 1];
        }
        // End LINEAR_REGRESSION

        return output
    }

    var tsf_1 = tsf;

    /**
     * 
     * @param {number[]} source 
     * @param {number} y_period 
     * @param {number} z_period 
     * @param {number} [size]
     * @returns 
     */
    async function tsi(
        source, y_period,
        z_period, size = source.length
    ) {

        const output = [];

        // if (y_period < 1 || z_period < 1) {
        //     throw new Error("Invalid Options")
        // }

        let progress = -1;

        let price = 0;
        let y_ema_num = 0;
        let z_ema_num = 0;
        let y_ema_den = 0;
        let z_ema_den = 0;

        let i = 0;
        for (; i < size && progress == -1; ++i, ++progress) {
            price = source[i];
        }

        for (; i < size && progress == 0; ++i, ++progress) {
            z_ema_num = y_ema_num = source[i] - price;
            z_ema_den = y_ema_den = Math.abs(source[i] - price);

            output.push(100 * (z_ema_den ? z_ema_num / z_ema_den : 0));

            price = source[i];
        }

        for (; i < size; ++i, ++progress) {
            y_ema_num = ((source[i] - price) - y_ema_num) * 2 / (1 + y_period) + y_ema_num;
            y_ema_den = ((Math.abs(source[i] - price)) - y_ema_den) * 2 / (1 + y_period) + y_ema_den;

            z_ema_num = (y_ema_num - z_ema_num) * 2 / (1 + z_period) + z_ema_num;
            z_ema_den = (y_ema_den - z_ema_den) * 2 / (1 + z_period) + z_ema_den;


            output.push(100 * (z_ema_den ? z_ema_num / z_ema_den : 0));

            price = source[i];
        }

        return output
    }

    var tsi_1 = tsi;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} [size]
     * @returns 
     */
    async function typprice(
        high, low,
        close, size = high.length
    ) {

        const output = [];

        for (let i = 0; i < size; ++i) {
            output.push((high[i] + low[i] + close[i]) * (1.0 / 3.0));
        }

        return output
    }

    var typprice_1 = typprice;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} short_period 
     * @param {number} medium_period 
     * @param {number} long_period 
     * @param {number} [size]
     * @returns 
     */
    async function ultosc(
        high, low,
        close, short_period,
        medium_period, long_period,
        size = high.length
    ) {

        const output = [];

        // if (short_period < 1) throw new Error("Invalid Options")
        // if (medium_period < short_period) throw new Error("Invalid Options")
        // if (long_period < medium_period) throw new Error("Invalid Options")
        // if (size <= medium_period) throw new Error("Out of range")


        const bp_buf = {
            size: long_period,
            index: 0,
            pushes: 0,
            sum: 0,
            vals: []
        };

        const r_buf = {
            size: long_period,
            index: 0,
            pushes: 0,
            sum: 0,
            vals: []
        };

        let bp_short_sum = 0;
        let bp_medium_sum = 0;
        let r_short_sum = 0;
        let r_medium_sum = 0;

        for (let i = 1; i < size; ++i) {

            // Start MIN
            // MIN(a,b) ((a)<(b)?(a):(b))
            const true_low = ((low[i]) < (close[i - 1]) ? (low[i]) : (close[i - 1]));
            // End MIN

            // Start MAX
            // MAX(a,b) ((a)>(b)?(a):(b))
            const true_high = ((high[i]) > (close[i - 1]) ? (high[i]) : (close[i - 1]));
            // End MAX

            const bp = close[i] - true_low;
            const r = true_high - true_low;

            bp_short_sum += bp;
            bp_medium_sum += bp;
            r_short_sum += r;
            r_medium_sum += r;

            // Start ti_buffer_push
            // BUFFER = bp_buf
            // VAL    = bp
            // ti_buffer_push(bp_buf, bp)
            if (bp_buf.pushes >= bp_buf.size) {
                bp_buf.sum -= bp_buf.vals[bp_buf.index];
            }

            bp_buf.sum += bp;
            bp_buf.vals[bp_buf.index] = bp;
            bp_buf.pushes += 1;
            bp_buf.index = (bp_buf.index + 1);
            if (bp_buf.index >= bp_buf.size) bp_buf.index = 0;
            // End ti_buffer_push

            // Start ti_buffer_push
            // BUFFER = r_buf
            // VAL    = r
            // ti_buffer_push(r_buf, r)
            if (r_buf.pushes >= r_buf.size) {
                r_buf.sum -= r_buf.vals[r_buf.index];
            }

            r_buf.sum += r;
            r_buf.vals[r_buf.index] = r;
            r_buf.pushes += 1;
            r_buf.index = (r_buf.index + 1);
            if (r_buf.index >= r_buf.size) r_buf.index = 0;
            // End ti_buffer_push

            if (i > short_period) {
                let short_index = bp_buf.index - short_period - 1;
                if (short_index < 0) short_index += long_period;

                bp_short_sum -= bp_buf.vals[short_index];
                r_short_sum -= r_buf.vals[short_index];

                if (i > medium_period) {
                    let medium_index = bp_buf.index - medium_period - 1;
                    if (medium_index < 0) medium_index += long_period;
                    bp_medium_sum -= bp_buf.vals[medium_index];
                    r_medium_sum -= r_buf.vals[medium_index];
                }
            }

            if (i >= long_period) {
                const first = 4 * bp_short_sum / r_short_sum;
                const second = 2 * bp_medium_sum / r_medium_sum;
                const third = 1 * bp_buf.sum / r_buf.sum;
                const ult = (first + second + third) * 100.0 / 7.0;
                output.push(ult);
            }
        }

        return output
    }

    var ultosc_1 = ultosc;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function varf(
        source, period,
        size = source.length
    ) {

        const output = [];

        const scale = 1.0 / period;

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let sum = 0;
        let sum2 = 0;

        for (let i = 0; i < period; ++i) {
            sum += source[i];
            sum2 += source[i] * source[i];
        }

        output.push(sum2 * scale - (sum * scale) * (sum * scale));

        for (let i = period; i < size; ++i) {
            sum += source[i];
            sum2 += source[i] * source[i];

            sum -= source[i - period];
            sum2 -= source[i - period] * source[i - period];

            output.push(sum2 * scale - (sum * scale) * (sum * scale));
        }

        return output
    }

    var varf_1 = varf;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function vhf(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        let trail = 1;
        let maxi = -1;
        let mini = -1;

        let max = source[0];
        let min = source[0];
        let bar;

        let sum = 0;

        let yc = source[0];
        let c;

        let j;
        for (let i = 1; i < period; ++i) {
            c = source[i];
            sum += Math.abs(c - yc);
            yc = c;
        }

        for (let i = period; i < size; ++i, ++trail) {
            c = source[i];
            sum += Math.abs(c - yc);
            yc = c;

            if (i > period) {
                sum -= Math.abs(source[i - period] - source[i - period - 1]);
            }

            // Maintain highest
            bar = c;
            if (maxi < trail) {
                maxi = trail;
                max = source[maxi];
                j = trail;

                while (++j <= i) {
                    bar = source[j];
                    if (bar >= max) {
                        max = bar;
                        maxi = j;
                    }
                }

            } else if (bar >= max) {
                maxi = i;
                max = bar;
            }

            // Maintain lowest
            bar = c;
            if (mini < trail) {
                mini = trail;
                min = source[mini];
                j = trail;

                while (++j <= i) {
                    bar = source[j];
                    if (bar <= min) {
                        min = bar;
                        mini = j;
                    }
                }

            } else if (bar <= min) {
                mini = i;
                min = bar;
            }

            output.push(Math.abs(max - min) / sum);
        }

        return output
    }

    var vhf_1 = vhf;

    /**
     * 
     * @param {number[]} source 
     * @param {number} short_period 
     * @param {number} long_period 
     * @param {number} alpha 
     * @param {number} [size]
     * @returns 
     */
    async function vidya(
        source, short_period,
        long_period, alpha,
        size = source.length
    ) {

        const output = [];

        const short_div = 1.0 / short_period;
        const long_div = 1.0 / long_period;

        // if (short_period < 1) throw new Error("Invalid Options")
        // if (long_period < short_period) throw new Error("Invalid Options")
        // if (long_period < 2) throw new Error("Invalid Options")
        // if (alpha < 0.0 || alpha > 1.0) throw new Error("Invalid Options")
        // if (size <= long_period - 2) throw new Error("Out of range")

        let short_sum = 0;
        let short_sum2 = 0;

        let long_sum = 0;
        let long_sum2 = 0;

        for (let i = 0; i < long_period; ++i) {
            long_sum += source[i];
            long_sum2 += source[i] * source[i];

            if (i >= long_period - short_period) {
                short_sum += source[i];
                short_sum2 += source[i] * source[i];
            }
        }

        let val = source[long_period - 2];
        output.push(val);

        if (long_period - 1 < size) {
            const short_stddev = Math.sqrt(short_sum2 * short_div - (short_sum * short_div) * (short_sum * short_div));
            const long_stddev = Math.sqrt(long_sum2 * long_div - (long_sum * long_div) * (long_sum * long_div));
            let k = short_stddev / long_stddev;
            if (k != k) k = 0; /* In some conditions it works out that we take the sqrt(-0.0), which gives NaN.
                                      That implies that k should be zero. */
            k *= alpha;
            val = (source[long_period - 1] - val) * k + val;
            output.push(val);
        }

        for (let i = long_period; i < size; ++i) {
            long_sum += source[i];
            long_sum2 += source[i] * source[i];

            short_sum += source[i];
            short_sum2 += source[i] * source[i];

            long_sum -= source[i - long_period];
            long_sum2 -= source[i - long_period] * source[i - long_period];

            short_sum -= source[i - short_period];
            short_sum2 -= source[i - short_period] * source[i - short_period];

            const short_stddev = Math.sqrt(short_sum2 * short_div - (short_sum * short_div) * (short_sum * short_div));
            const long_stddev = Math.sqrt(long_sum2 * long_div - (long_sum * long_div) * (long_sum * long_div));
            let k = short_stddev / long_stddev;
            if (k != k) k = 0;
            k *= alpha;
            val = (source[i] - val) * k + val;

            output.push(val);
        }

        return output
    }

    var vidya_1 = vidya;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function volatility(
        source, period,
        size = source.length
    ) {

        const output = [];

        const scale = 1.0 / period;
        const annual = Math.sqrt(252); /* Multiplier, number of trading days in year. */

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period) throw new Error("Out of range")

        let sum = 0;
        let sum2 = 0;

        for (let i = 1; i <= period; ++i) {
            // Start Change()
            const c = (source[i] / source[i - 1] - 1.0);
            // End Change()
            sum += c;
            sum2 += c * c;
        }

        output.push(Math.sqrt(sum2 * scale - (sum * scale) * (sum * scale)) * annual);

        for (let i = period + 1; i < size; ++i) {
            // Start Change()
            const c = (source[i] / source[i - 1] - 1.0);
            // End Change()
            sum += c;
            sum2 += c * c;

            // Start Change()
            const cp = (source[i - period] / source[i - period - 1] - 1.0);
            // Start Change()
            sum -= cp;
            sum2 -= cp * cp;

            output.push(Math.sqrt(sum2 * scale - (sum * scale) * (sum * scale)) * annual);
        }

        return output
    }

    var volatility_1 = volatility;

    /**
     * 
     * @param {number[]} source 
     * @param {number} short_period 
     * @param {number} long_period 
     * @param {number} [size]
     * @returns 
     */
    async function vosc(
        source, short_period,
        long_period, size = source.length
    ) {

        const output = [];

        const short_div = 1.0 / short_period;
        const long_div = 1.0 / long_period;

        // if (short_period < 1) throw new Error("Invalid Options")
        // if (long_period < short_period) throw new Error("Invalid Options")
        // if (size <= long_period - 1) throw new Error("Out of range")

        let short_sum = 0;
        let long_sum = 0;

        for (let i = 0; i < long_period; ++i) {
            if (i >= (long_period - short_period)) {
                short_sum += source[i];
            }
            long_sum += source[i];
        }

        const savg = short_sum * short_div;
        const lavg = long_sum * long_div;
        output.push(100.0 * (savg - lavg) / lavg);

        for (let i = long_period; i < size; ++i) {
            short_sum += source[i];
            short_sum -= source[i - short_period];

            long_sum += source[i];
            long_sum -= source[i - long_period];

            const savg = short_sum * short_div;
            const lavg = long_sum * long_div;
            output.push(100.0 * (savg - lavg) / lavg);
        }

        return output
    }

    var vosc_1 = vosc;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number[]} volume 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function vwap(
        high, low,
        close, volume,
        period, size = high.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")

        let progress = -period + 1;

        let num = 0;
        let den = 0;

        let i = 0;
        for (; i < size && progress < 1; ++i, ++progress) {
            num += (high[i] + low[i] + close[i]) / 3 * volume[i];
            den += volume[i];
        }

        if (i > 0 && progress == 1) {
            output.push(num / den);
        }

        for (; i < size; ++i, ++progress) {
            num += (high[i] + low[i] + close[i]) / 3 * volume[i]
                - (high[i - period] + low[i - period] + close[i - period]) / 3 * volume[i - period];
            den += volume[i] - volume[i - period];

            output.push(num / den);
        }

        return output
    }

    var vwap_1 = vwap;

    /**
     * 
     * @param {number[]} source 
     * @param {number[]} volume 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function vwma(
        source, volume,
        period, size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let sum = 0;
        let vsum = 0;

        for (let i = 0; i < period; ++i) {
            sum += source[i] * volume[i];
            vsum += volume[i];
        }

        output.push(sum / vsum);

        for (let i = period; i < size; ++i) {
            sum += source[i] * volume[i];
            sum -= source[i - period] * volume[i - period];
            vsum += volume[i];
            vsum -= volume[i - period];

            output.push(sum / vsum);
        }

        return output
    }

    var vwma_1 = vwma;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} [size]
     * @returns 
     */
    async function wad(
        high, low,
        close, size = high.length
    ) {

        // if (size <= 1) throw new Error("Out of range")

        const output = [];
        let sum = 0;
        let yc = close[0];

        for (let i = 1; i < size; ++i) {
            const c = close[i];

            if (c > yc) {
                // Start MIN
                // MIN(a,b) ((a)<(b)?(a):(b))
                sum += c - ((yc) < (low[i]) ? (yc) : (low[i]));
                // End MIN()
            } else if (c < yc) {
                // Start MAX
                // MAX(a,b) ((a)>(b)?(a):(b))
                sum += c - ((yc) > (high[i]) ? (yc) : (high[i]));
                // End MAX
            } else ;

            output.push(sum);

            yc = close[i];
        }

        return output
    }

    var wad_1 = wad;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} [size]
     * @returns 
     */
    async function wcprice(
        high, low,
        close, size = high.length
    ) {

        const output = [];

        for (let i = 0; i < size; ++i) {
            output.push((high[i] + low[i] + close[i] + close[i]) * 0.25);
        }

        return output
    }

    var wcprice_1 = wcprice;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function wilders(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        const per = 1.0 / (period);

        let sum = 0;

        for (let i = 0; i < period; ++i) {
            sum += source[i];
        }


        let val = sum / period;
        output.push(val);

        for (let i = period; i < size; ++i) {
            val = (source[i] - val) * per + val;
            output.push(val);
        }

        return output
    }

    var wilders_1 = wilders;

    /**
     * 
     * @param {number[]} high 
     * @param {number[]} low 
     * @param {number[]} close 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function willr(
        high, low,
        close, period,
        size = high.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        let trail = 0;
        let maxi = -1;
        let mini = -1;
        let max = high[0];
        let min = low[0];
        let bar;

        let j;
        for (let i = period - 1; i < size; ++i, ++trail) {

            // Maintain highest
            bar = high[i];
            if (maxi < trail) {
                maxi = trail;
                max = high[maxi];
                j = trail;

                while (++j <= i) {
                    bar = high[j];
                    if (bar >= max) {
                        max = bar;
                        maxi = j;
                    }
                }

            } else if (bar >= max) {
                maxi = i;
                max = bar;
            }


            // Maintain lowest
            bar = low[i];
            if (mini < trail) {
                mini = trail;
                min = low[mini];
                j = trail;

                while (++j <= i) {
                    bar = low[j];
                    if (bar <= min) {
                        min = bar;
                        mini = j;
                    }
                }

            } else if (bar <= min) {
                mini = i;
                min = bar;
            }


            const highlow = (max - min);
            const r = highlow == 0.0 ? 0.0 : -100 * ((max - close[i]) / highlow);
            output.push(r);
        }

        return output
    }

    var willr_1 = willr;

    /**
     * 
     * @param {number[]} source 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function wma(
        source, period,
        size = source.length
    ) {

        const output = [];

        // if (period < 1) throw new Error("Invalid Options")
        // if (size <= period - 1) throw new Error("Out of range")

        const weights = period * (period + 1) / 2;

        let sum = 0;
        let weight_sum = 0;

        for (let i = 0; i < period - 1; ++i) {
            weight_sum += source[i] * (i + 1);
            sum += source[i];
        }

        for (let i = period - 1; i < size; ++i) {
            weight_sum += source[i] * period;
            sum += source[i];

            output.push(weight_sum / weights);

            weight_sum -= sum;
            sum -= source[i - period + 1];
        }

        return output
    }

    var wma_1 = wma;

    /**
     * 
     * @param {number[]} input 
     * @param {number} period 
     * @param {number} [size]
     * @returns 
     */
    async function zlema(input, period, size = input.length) {

        const lag = Math.floor((period - 1) / 2);

        const output = [];

        const per = 2 / (period + 1);

        let val = input[lag - 1];
        output.push(val);

        let i;
        for (i = lag; i < size; ++i) {
            const c = input[i];
            const l = input[i - lag];

            val = ((c + (c - l)) - val) * per + val;
            output.push(val);
        }

        return output
    }

    var zlema_1 = zlema;

    var indicatorsJs = {
        _cvi: _cvi_1,
        abands: abands_1,
        ad: ad_1,
        adosc: adosc_1,
        adx: adx_1,
        adxr: adxr_1,
        alma: alma_1,
        ao: ao_1,
        apo: apo_1,
        aroon: aroon_1,
        aroonosc: aroonosc_1,
        atr: atr_1,
        avgprice: avgprice_1,
        bbands: bbands_1,
        bop: bop_1,
        cci: cci_1,
        ce: ce_1,
        cmf: cmf_1,
        cmo: cmo_1,
        copp: copp_1,
        crossany: crossany_1,
        crossover: crossover_1,
        crossOverNumber: crossOverNumber_1,
        crossUnderNumber: crossUnderNumber_1,
        cvi: cvi_1,
        dc: dc_1,
        decay: decay_1,
        dema: dema_1,
        di: di_1,
        dm: dm_1,
        dpo: dpo_1,
        dx: dx_1,
        edecay: edecay_1,
        ema: ema_1,
        emv: emv_1,
        fi: fi_1,
        fisher: fisher_1,
        fosc: fosc_1,
        hma: hma_1,
        kama: kama_1,
        kc: kc_1,
        kst: kst_1,
        kvo: kvo_1,
        lag: lag_1,
        linreg: linreg_1,
        linregintercept: linregintercept_1,
        linregslope: linregslope_1,
        macd: macd_1,
        marketfi: marketfi_1,
        mass: mass_1,
        max: max_1,
        md: md_1,
        medprice: medprice_1,
        mfi: mfi_1,
        min: min_1,
        mom: mom_1,
        msw: msw_1,
        natr: natr_1,
        normalize: normalize_1,
        normalize2: normalize2_1,
        nvi: nvi_1,
        obv: obv_1,
        pbands: pbands_1,
        pfe: pfe_1,
        posc: posc_1,
        ppo: ppo_1,
        psar: psar_1,
        pvi: pvi_1,
        qstick: qstick_1,
        rmi: rmi_1,
        rmta: rmta_1,
        roc: roc_1,
        rocr: rocr_1,
        rsi: rsi_1,
        rvi: rvi_1,
        sma: sma_1,
        smi: smi_1,
        stddev: stddev_1,
        stderr: stderr_1,
        stoch: stoch_1,
        stochrsi: stochrsi_1,
        sum: sum_1,
        tema: tema_1,
        tr: tr_1,
        trima: trima_1,
        trix: trix_1,
        tsf: tsf_1,
        tsi: tsi_1,
        typprice: typprice_1,
        ultosc: ultosc_1,
        varf: varf_1,
        vhf: vhf_1,
        vidya: vidya_1,
        volatility: volatility_1,
        vosc: vosc_1,
        vwap: vwap_1,
        vwma: vwma_1,
        wad: wad_1,
        wcprice: wcprice_1,
        wilders: wilders_1,
        willr: willr_1,
        wma: wma_1,
        zlema: zlema_1,
    };
    var indicatorsJs_1 = indicatorsJs._cvi;
    var indicatorsJs_2 = indicatorsJs.abands;
    var indicatorsJs_3 = indicatorsJs.ad;
    var indicatorsJs_4 = indicatorsJs.adosc;
    var indicatorsJs_5 = indicatorsJs.adx;
    var indicatorsJs_6 = indicatorsJs.adxr;
    var indicatorsJs_7 = indicatorsJs.alma;
    var indicatorsJs_8 = indicatorsJs.ao;
    var indicatorsJs_9 = indicatorsJs.apo;
    var indicatorsJs_10 = indicatorsJs.aroon;
    var indicatorsJs_11 = indicatorsJs.aroonosc;
    var indicatorsJs_12 = indicatorsJs.atr;
    var indicatorsJs_13 = indicatorsJs.avgprice;
    var indicatorsJs_14 = indicatorsJs.bbands;
    var indicatorsJs_15 = indicatorsJs.bop;
    var indicatorsJs_16 = indicatorsJs.cci;
    var indicatorsJs_17 = indicatorsJs.ce;
    var indicatorsJs_18 = indicatorsJs.cmf;
    var indicatorsJs_19 = indicatorsJs.cmo;
    var indicatorsJs_20 = indicatorsJs.copp;
    var indicatorsJs_21 = indicatorsJs.crossany;
    var indicatorsJs_22 = indicatorsJs.crossover;
    var indicatorsJs_23 = indicatorsJs.crossOverNumber;
    var indicatorsJs_24 = indicatorsJs.crossUnderNumber;
    var indicatorsJs_25 = indicatorsJs.cvi;
    var indicatorsJs_26 = indicatorsJs.dc;
    var indicatorsJs_27 = indicatorsJs.decay;
    var indicatorsJs_28 = indicatorsJs.dema;
    var indicatorsJs_29 = indicatorsJs.di;
    var indicatorsJs_30 = indicatorsJs.dm;
    var indicatorsJs_31 = indicatorsJs.dpo;
    var indicatorsJs_32 = indicatorsJs.dx;
    var indicatorsJs_33 = indicatorsJs.edecay;
    var indicatorsJs_34 = indicatorsJs.ema;
    var indicatorsJs_35 = indicatorsJs.emv;
    var indicatorsJs_36 = indicatorsJs.fi;
    var indicatorsJs_37 = indicatorsJs.fisher;
    var indicatorsJs_38 = indicatorsJs.fosc;
    var indicatorsJs_39 = indicatorsJs.hma;
    var indicatorsJs_40 = indicatorsJs.kama;
    var indicatorsJs_41 = indicatorsJs.kc;
    var indicatorsJs_42 = indicatorsJs.kst;
    var indicatorsJs_43 = indicatorsJs.kvo;
    var indicatorsJs_44 = indicatorsJs.lag;
    var indicatorsJs_45 = indicatorsJs.linreg;
    var indicatorsJs_46 = indicatorsJs.linregintercept;
    var indicatorsJs_47 = indicatorsJs.linregslope;
    var indicatorsJs_48 = indicatorsJs.macd;
    var indicatorsJs_49 = indicatorsJs.marketfi;
    var indicatorsJs_50 = indicatorsJs.mass;
    var indicatorsJs_51 = indicatorsJs.max;
    var indicatorsJs_52 = indicatorsJs.md;
    var indicatorsJs_53 = indicatorsJs.medprice;
    var indicatorsJs_54 = indicatorsJs.mfi;
    var indicatorsJs_55 = indicatorsJs.min;
    var indicatorsJs_56 = indicatorsJs.mom;
    var indicatorsJs_57 = indicatorsJs.msw;
    var indicatorsJs_58 = indicatorsJs.natr;
    var indicatorsJs_59 = indicatorsJs.normalize;
    var indicatorsJs_60 = indicatorsJs.normalize2;
    var indicatorsJs_61 = indicatorsJs.nvi;
    var indicatorsJs_62 = indicatorsJs.obv;
    var indicatorsJs_63 = indicatorsJs.pbands;
    var indicatorsJs_64 = indicatorsJs.pfe;
    var indicatorsJs_65 = indicatorsJs.posc;
    var indicatorsJs_66 = indicatorsJs.ppo;
    var indicatorsJs_67 = indicatorsJs.psar;
    var indicatorsJs_68 = indicatorsJs.pvi;
    var indicatorsJs_69 = indicatorsJs.qstick;
    var indicatorsJs_70 = indicatorsJs.rmi;
    var indicatorsJs_71 = indicatorsJs.rmta;
    var indicatorsJs_72 = indicatorsJs.roc;
    var indicatorsJs_73 = indicatorsJs.rocr;
    var indicatorsJs_74 = indicatorsJs.rsi;
    var indicatorsJs_75 = indicatorsJs.rvi;
    var indicatorsJs_76 = indicatorsJs.sma;
    var indicatorsJs_77 = indicatorsJs.smi;
    var indicatorsJs_78 = indicatorsJs.stddev;
    var indicatorsJs_79 = indicatorsJs.stderr;
    var indicatorsJs_80 = indicatorsJs.stoch;
    var indicatorsJs_81 = indicatorsJs.stochrsi;
    var indicatorsJs_82 = indicatorsJs.sum;
    var indicatorsJs_83 = indicatorsJs.tema;
    var indicatorsJs_84 = indicatorsJs.tr;
    var indicatorsJs_85 = indicatorsJs.trima;
    var indicatorsJs_86 = indicatorsJs.trix;
    var indicatorsJs_87 = indicatorsJs.tsf;
    var indicatorsJs_88 = indicatorsJs.tsi;
    var indicatorsJs_89 = indicatorsJs.typprice;
    var indicatorsJs_90 = indicatorsJs.ultosc;
    var indicatorsJs_91 = indicatorsJs.varf;
    var indicatorsJs_92 = indicatorsJs.vhf;
    var indicatorsJs_93 = indicatorsJs.vidya;
    var indicatorsJs_94 = indicatorsJs.volatility;
    var indicatorsJs_95 = indicatorsJs.vosc;
    var indicatorsJs_96 = indicatorsJs.vwap;
    var indicatorsJs_97 = indicatorsJs.vwma;
    var indicatorsJs_98 = indicatorsJs.wad;
    var indicatorsJs_99 = indicatorsJs.wcprice;
    var indicatorsJs_100 = indicatorsJs.wilders;
    var indicatorsJs_101 = indicatorsJs.willr;
    var indicatorsJs_102 = indicatorsJs.wma;
    var indicatorsJs_103 = indicatorsJs.zlema;

    exports._cvi = indicatorsJs_1;
    exports.abands = indicatorsJs_2;
    exports.ad = indicatorsJs_3;
    exports.adosc = indicatorsJs_4;
    exports.adx = indicatorsJs_5;
    exports.adxr = indicatorsJs_6;
    exports.alma = indicatorsJs_7;
    exports.ao = indicatorsJs_8;
    exports.apo = indicatorsJs_9;
    exports.aroon = indicatorsJs_10;
    exports.aroonosc = indicatorsJs_11;
    exports.atr = indicatorsJs_12;
    exports.avgprice = indicatorsJs_13;
    exports.bbands = indicatorsJs_14;
    exports.bop = indicatorsJs_15;
    exports.cci = indicatorsJs_16;
    exports.ce = indicatorsJs_17;
    exports.cmf = indicatorsJs_18;
    exports.cmo = indicatorsJs_19;
    exports.copp = indicatorsJs_20;
    exports.crossOverNumber = indicatorsJs_23;
    exports.crossUnderNumber = indicatorsJs_24;
    exports.crossany = indicatorsJs_21;
    exports.crossover = indicatorsJs_22;
    exports.cvi = indicatorsJs_25;
    exports.dc = indicatorsJs_26;
    exports.decay = indicatorsJs_27;
    exports.default = indicatorsJs;
    exports.dema = indicatorsJs_28;
    exports.di = indicatorsJs_29;
    exports.dm = indicatorsJs_30;
    exports.dpo = indicatorsJs_31;
    exports.dx = indicatorsJs_32;
    exports.edecay = indicatorsJs_33;
    exports.ema = indicatorsJs_34;
    exports.emv = indicatorsJs_35;
    exports.fi = indicatorsJs_36;
    exports.fisher = indicatorsJs_37;
    exports.fosc = indicatorsJs_38;
    exports.hma = indicatorsJs_39;
    exports.kama = indicatorsJs_40;
    exports.kc = indicatorsJs_41;
    exports.kst = indicatorsJs_42;
    exports.kvo = indicatorsJs_43;
    exports.lag = indicatorsJs_44;
    exports.linreg = indicatorsJs_45;
    exports.linregintercept = indicatorsJs_46;
    exports.linregslope = indicatorsJs_47;
    exports.macd = indicatorsJs_48;
    exports.marketfi = indicatorsJs_49;
    exports.mass = indicatorsJs_50;
    exports.max = indicatorsJs_51;
    exports.md = indicatorsJs_52;
    exports.medprice = indicatorsJs_53;
    exports.mfi = indicatorsJs_54;
    exports.min = indicatorsJs_55;
    exports.mom = indicatorsJs_56;
    exports.msw = indicatorsJs_57;
    exports.natr = indicatorsJs_58;
    exports.normalize = indicatorsJs_59;
    exports.normalize2 = indicatorsJs_60;
    exports.nvi = indicatorsJs_61;
    exports.obv = indicatorsJs_62;
    exports.pbands = indicatorsJs_63;
    exports.pfe = indicatorsJs_64;
    exports.posc = indicatorsJs_65;
    exports.ppo = indicatorsJs_66;
    exports.psar = indicatorsJs_67;
    exports.pvi = indicatorsJs_68;
    exports.qstick = indicatorsJs_69;
    exports.rmi = indicatorsJs_70;
    exports.rmta = indicatorsJs_71;
    exports.roc = indicatorsJs_72;
    exports.rocr = indicatorsJs_73;
    exports.rsi = indicatorsJs_74;
    exports.rvi = indicatorsJs_75;
    exports.sma = indicatorsJs_76;
    exports.smi = indicatorsJs_77;
    exports.stddev = indicatorsJs_78;
    exports.stderr = indicatorsJs_79;
    exports.stoch = indicatorsJs_80;
    exports.stochrsi = indicatorsJs_81;
    exports.sum = indicatorsJs_82;
    exports.tema = indicatorsJs_83;
    exports.tr = indicatorsJs_84;
    exports.trima = indicatorsJs_85;
    exports.trix = indicatorsJs_86;
    exports.tsf = indicatorsJs_87;
    exports.tsi = indicatorsJs_88;
    exports.typprice = indicatorsJs_89;
    exports.ultosc = indicatorsJs_90;
    exports.varf = indicatorsJs_91;
    exports.vhf = indicatorsJs_92;
    exports.vidya = indicatorsJs_93;
    exports.volatility = indicatorsJs_94;
    exports.vosc = indicatorsJs_95;
    exports.vwap = indicatorsJs_96;
    exports.vwma = indicatorsJs_97;
    exports.wad = indicatorsJs_98;
    exports.wcprice = indicatorsJs_99;
    exports.wilders = indicatorsJs_100;
    exports.willr = indicatorsJs_101;
    exports.wma = indicatorsJs_102;
    exports.zlema = indicatorsJs_103;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
